<?php

namespace App\Http\Controllers\Admin;

use App\Models\Log;
use Backpack\CRUD\app\Http\Controllers\CrudController;

// VALIDATION: change the requests to match your own file names if you need form validation
use App\Http\Requests\StatementRequest as StoreRequest;
use App\Http\Requests\StatementRequest as UpdateRequest;
use Backpack\CRUD\CrudPanel;
use Illuminate\Database\Eloquent\Builder;
use Illuminate\Support\Arr;
use Auth;
use App;
use DB;

use App\Models\Specialisation;
use App\Models\Subject;
use App\Models\Bachelor as Statement;
use App\Models\Group;

use App\Models\Mark;
use App\Models\MarkGroup;

use Illuminate\Http\Request;
use Illuminate\Support\Carbon;

/**
 * Class StatementCrudController
 * @package App\Http\Controllers\Admin
 * @property-read CrudPanel $crud
 */
class ArchiveStatementCrudController extends CrudController
{

    private const FORM = 'statement';


    public function setup()
    {
        App::setLocale(session('language', 'ka'));
        /*
        |--------------------------------------------------------------------------
        | CrudPanel Basic Information
        |-------------------------------------------------------------------------
        */
        $this->crud->setModel('App\Models\ArchiveStatement');
        $this->crud->setRoute(config('backpack.base.route_prefix') . '/archive_statement');
        $this->crud->setEntityNameStrings(__('base.' . self::FORM .'s_add.single'), __('base.' . self::FORM . 's_add.plural'));

        /*
        |--------------------------------------------------------------------------
        | CrudPanel Configuration
        |--------------------------------------------------------------------------
        */


        //query edit

        $this->crud->query->orderBy('created_at');
        $this->crud->query->whereIn('form', Statement::ALL_FORMS);

        $this->crud->query->where(function ($quer){
            $quer->whereNull('closed_at')->orWhere('closed_at', '<', Carbon::now());
        });

        //query end

        if (!empty(session('faculty'))) {
            $this->crud->addClause('whereHas', 'specialisation', function ($query) {
                $query->where('faculty_id', '=', session('faculty'));
            });
        }
        $this->crud->addClause('whereHas', 'specialisation', function ($query) {
            $query->whereNull('closed_at')->orWhere('closed_at', '<', Carbon::now());
          //  $query->where('faculty_id', '!=', 5);
        });



        if (Auth::user()->hasRole('Lecturer')) {
            $this->crud->addClause('whereDate', 'closed_at', '>=', Carbon::today());
            $this->crud->addClause('whereHas', 'lecturers', function ($query) {
                $query->where('id', Auth::user()->id);
            });
        }
        if (!empty(session('faculty'))) {
            $specialisations = Specialisation::all()
                ->pluck('name', 'id')
                ->toArray();
            $group = Group::whereHas('specialisation', function ($query) {
                $query->where('faculty_id', session('faculty'));
            })
                ->get()
                ->pluck('name', 'id')
                ->toArray();
            $subjects = Subject::all()
                ->pluck('name', 'id')
                ->toArray();
        } else {
            $specialisations = Specialisation::all()
                ->pluck('name', 'id')
                ->toArray();
            $group = Group::all()
                ->pluck('name', 'id')
                ->toArray();
            $subjects = Subject::all()
                ->pluck('name', 'id')
                ->toArray();
        }

       // $empty_subjects = Subject::whereDoesntHave('faculties')->get()
        //    ->pluck('name', 'id')
        //    ->toArray();
       // $subjects += $empty_subjects;

        $this->crud->addColumn([
            'name' => 'number',
            'label' => __('base.statement.number'),
        ]);

        $this->crud->addColumn([
            'name' => 'subject_id',
            'label' => __('base.statement.subject'),
            'type' => 'select_from_array',
            'options' => $subjects
        ]);

        $this->crud->addColumn([
            'name' => 'group_id',
            'label' => __('base.statement.group'),
            'type' => 'select_from_array',
            'options' => $group
        ]);

        $this->crud->addColumn([
            'name' => 'closed_at',
            'label' => __('base.statement.closed_at'),
            'type' => 'date',
        ]);

        $this->crud->addColumn([
            'name' => 'lecturers',
            'label' => __('base.statement.lecturers'),
            'type' => "select_multiple",
            'entity' => 'lecturers',
            'attribute' => 'full_name',
            'model' => "App\Models\Lecturer"
        ]);


        if (!Auth::user()->hasAnyRole(['SuperAdmin', 'Coordinator'])) {
            $this->crud->denyAccess('create');
            $this->crud->removeButton('create');
        }
       
       $boolean = DB::table('edit_archive_statement')->first()->boolean;
     
        if($boolean == 0){
            $this->crud->denyAccess('update');
            $this->crud->removeButton('update');
        }
        if (!Auth::user()->hasAnyRole(['SuperAdmin', 'Coordinator'])) {
            $this->crud->denyAccess('delete');
            $this->crud->removeButton('delete');
        }
        if (Auth::user()->hasAnyRole(['SuperAdmin', 'Coordinator', 'Lecturer'])) {
            $this->crud->addButtonFromModelFunction('line', 'print', 'print', 'beginning');
        }
        if (Auth::user()->hasAnyRole(['SuperAdmin', 'Coordinator', 'Lecturer'])) {
            $this->crud->addButtonFromModelFunction('line', 'edit', 'edit', 'beginning');
        }
        
        if (Auth::user()->hasAnyRole(['SuperAdmin', 'Coordinator'])) {
            $this->crud->addButtonFromModelFunction('line', 'clone', 'cloneStatement', 'beginning');
        }

        $this->crud->addField([
            'name' => 'number',
            'label' => __('base.statement.number'),
            'type' => 'text',
            'wrapperAttributes' => [
                'class' => 'form-group col-md-12'
            ],
            'tab' => __('base.statement.main'),
        ]);

        $this->crud->addField([
            'name' => 'specialisation_id',
            'label' => __('base.statement.specialisation'),
            'type' => 'select2_from_array',
            'options' => $specialisations,
            'wrapperAttributes' => [
                'class' => 'form-group col-md-4'
            ],
            'tab' => __('base.statement.main'),
        ]);

        $this->crud->addField([
            'name' => 'group_id',
            'label' => __('base.statement.group'),
            'type' => 'select2_from_array',
            'options' => $group,
            'allows_null' => true,
            'wrapperAttributes' => [
                'class' => 'form-group col-md-4'
            ],
            'tab' => __('base.statement.main'),
        ]);

        $this->crud->addField([
            'name' => 'subject_id',
            'label' => __('base.statement.subject'),
            'type' => 'select2_from_array',
            'options' => $subjects,
            'wrapperAttributes' => [
                'class' => 'form-group col-md-4'
            ],
            'tab' => __('base.statement.main'),
        ]);

        $this->crud->addField([
            'name' => 'count',
            'label' => __('base.statement.count'),
            'type' => 'text',
            'wrapperAttributes' => [
                'class' => 'form-group col-md-6'
            ],
            'tab' => __('base.statement.main'),
        ]);

        $this->crud->addField([
            'name' => 'hours',
            'label' => __('base.statement.hours'),
            'type' => 'text',
            'wrapperAttributes' => [
                'class' => 'form-group col-md-6'
            ],
            'tab' => __('base.statement.main'),
        ]);

        $this->crud->addField([
            'name' => 'required',
            'label' => __('base.statement.required'),
            'type' => 'select2_from_array',
            'wrapperAttributes' => [
                'class' => 'form-group col-md-4'
            ],
            'options' => Arr::pluck(json_decode(config('settings.required'), true), 'name', 'slug'),
            'tab' => __('base.statement.main'),
        ]);

        $this->crud->addField([
            'name' => 'type',
            'label' => __('base.statement.type'),
            'type' => 'select2_from_array',
            'wrapperAttributes' => [
                'class' => 'form-group col-md-4'
            ],
            'options' => Arr::pluck(json_decode(config('settings.type'), true), 'name', 'slug'),
            'tab' => __('base.statement.main'),
        ]);

        $this->crud->addField([
            'name' => 'step',
            'label' => __('base.statement.step'),
            'type' => 'select2_from_array',
            'wrapperAttributes' => [
                'class' => 'form-group col-md-4'
            ],
            'options' => Arr::pluck(json_decode(config('settings.stage'), true), 'name', 'slug'),
            'tab' => __('base.statement.main'),
        ]);

        $this->crud->addField([
            'name' => 'semester',
            'label' => __('base.statement.semester'),
            'type' => 'select2_from_array',
            'wrapperAttributes' => [
                'class' => 'form-group col-md-6'
            ],
            'options' => Arr::pluck(json_decode(config('settings.semester'), true), 'name', 'slug'),
            'tab' => __('base.statement.main'),
        ]);

        $this->crud->addField([
            'name' => 'closed_at',
            'label' => __('base.statement.closed_at'),
            'type' => 'date',
            'wrapperAttributes' => [
                'class' => 'form-group col-md-6'
            ],
            'tab' => __('base.statement.main'),
        ]);

        $this->crud->addField([
            'name' => 'lecturers',
            'label' => __('base.statement.lecturers'),
            'type' => 'select2_multiple',
            'entity' => 'lecturers',
            'attribute' => 'full_name',
            'model' => 'App\Models\Lecturer',
            'pivot' => true,
            'options' => (function ($query) {
                return $query->get();
            }),
            'tab' => __('base.statement.users'),
            'fake' => true
        ]);

        $this->crud->addField([
            'name' => 'groups',
            'label' => __('base.statement.groups'),
            'type' => 'select2_from_array',
            'options' => $group,
            'allows_multiple' => true,
            'tab' => __('base.statement.users'),
            'fake' => true
        ]);

        $field_type = 'checkbox_inside_select';

        if ($ids = Auth::user()->coordinatorAddClause()) {
            $field_type = 'select2_from_ajax_multiple';
        }
         $field_type = 'select2_from_ajax_multiple';

        $this->crud->addField([
            'name' => 'students',
            'label' => __('base.statement.students'),
            'type' => $field_type,
            'entity' => 'students',
            'attribute' => 'full_name',
            'model' => "App\Models\Group",
            'pivot' => true,
            'options' => (function ($query) {
                if ($ids = Auth::user()->coordinatorAddClause()) {
                    return $query->whereHas('specialisations', function ($query) use ($ids) {
                        $query->whereHas('faculty', function ($query) use ($ids) {
                            $query->where('id', $ids)->get();
                        });
                    });

                } else {
                    $student_with_groups = App\Models\Group::with('students')->get();
                    $other_students = App\Models\Student::doesntHave('groups')->get();
                    $other = collect();
                    $other->put('name', 'Other');
                    $other->put('students', $other_students);
                    $student_with_groups->push($other);

                    return $student_with_groups;
                }
            }),
            'data_source' => backpack_url("getStudents"),
            'placeholder' => __('base.select_students'),
            'minimum_input_length' => 0,
            'tab' => __('base.statement.users'),
            'select_all' => true,
            'fake' => true,
        ]);

        $this->crud->addFilter([
            'type' => 'text',
            'name' => 'number',
            'label' => __('base.statement.number')
        ],
            false,
            function ($value) { // if the filter is active
                $this->crud->addClause('where', 'number', 'LIKE', "%$value%");
            });
            $all_forms = ['all' => __('base.statement.all')];
            foreach(Statement::ALL_FORMS as $value){
                $all_forms[$value] = __('base.rating.' . $value);
            }
         $this->crud->addFilter([
            'name' => 'form',
            'type' => 'select2',
            'label' => __('base.statement.form'),
        ],
            $all_forms,
            function ($value) {
                if($value == 'all') return;
                $this->crud->query = $this->crud->query->where('form', $value);
            });

        $this->crud->addFilter([
            'type' => 'date_range',
            'name' => 'closed_at',
            'label' => __('base.statement.closed_at')
        ],
            false,
            function ($value) { // if the filter is active, apply these constraints
                $dates = json_decode($value);
                $this->crud->addClause('where', 'closed_at', '>=', $dates->from);
                $this->crud->addClause('where', 'closed_at', '<=', $dates->to . ' 23:59:59');
            });

        $this->crud->addFilter([
            'name' => 'lecturers',
            'type' => 'text',
            'label' => __('base.lector')
        ],
            false,
            function ($value) {
                $this->crud->query = $this->crud->query->whereHas('lecturers', function (Builder $query) use ($value) {
                    $query->where('first_name', 'LIKE', "%$value%")
                        ->orWhere('last_name', 'LIKE', "%$value%");
                });
            });

        $this->crud->addFilter([
            'name' => 'email',
            'type' => 'text',
            'label' => __('base.email')
        ],
            false,
            function ($value) {
                $this->crud->query = $this->crud->query->whereHas('students', function (Builder $query) use ($value) {
                    $query->where('email', 'like', "%$value%");
                });
            });

        $this->crud->addFilter([
            'name' => 'subject',
            'type' => 'select2',
            'label' => __('base.statement.subject'),
        ],
            $subjects,
            function ($value) {
                $this->crud->query = $this->crud->query->where('subject_id', $value);
            });
            
            

        $this->crud->addFilter([
            'name' => 'passport',
            'type' => 'text',
            'label' => __('base.passport')
        ],
            false,
            function ($value) {
                $this->crud->query = $this->crud->query->whereHas('students', function (Builder $query) use ($value) {
                    $query->whereHas('user_attributes', function (Builder $query) use ($value) {
                        $query->where('value', $value);
                    });
                });
            });

        // add asterisk for fields that are required in StatementRequest
        $this->crud->setRequiredFields(StoreRequest::class, 'create');
        $this->crud->setRequiredFields(UpdateRequest::class, 'edit');
    }

    public function store(StoreRequest $request)
    {
        return back();
    }

    public function update(UpdateRequest $request)
    {
        if (!$request->user()->hasAnyRole(['SuperAdmin'])) {
            return back();
        }
        $redirect_location = parent::updateCrud($request);

        if (!empty($request->get('students'))) {
            $this->crud->entry->students()->sync($this->getSync('students', 'student'));
        }

        if (!empty($request->get('lecturers'))) {
            $this->crud->entry->lecturers()->sync($this->getSync('lecturers', 'lecturer'));
        }

        return $redirect_location;
    }


    public function getSync($key, $type)
    {
        $array = [];
        foreach (request()->get($key) as $item) {
            $array[$item] = ['type' => $type];
        }
        return $array;
    }

    public function rating($id)
    {
        /**
         * @var  Builder $statement
         */
        $statement = App\Models\Statement::with(['students' => function ($query) {
            $query->with(['user_attributes', 'marks'])->orderBy('last_name->ru', 'asc');
        }, 'lecturers', 'supervision' => function($query) use($id){
            $query->where('statement_id', $id);
        }])->findOrFail($id);
        if($statement->form != 'statement')
            return view('admin.rating_' . $statement->form, compact('statement'
            ));
        return view('admin.rating', compact('statement'
            ));
    }

    public function ratingSave(Request $request, $id)
    {
        
        $gropus = MarkGroup::all();
        $statement = Statement::with(['subject', 'students'])->where('form', self::FORM)->find($id);
        Mark::whereIn('user_id', $statement->students->pluck('id')->toArray())->where('statement_id', $statement->id)->delete();


        foreach ($request->get('marks') as $key => $value) {
            $save = [];
            foreach ($value as $slug => $item) {
                $mark = [];
                $mark['user_id'] = $key;
                $mark['subject_id'] = $statement->subject->id;
                $mark['statement_id'] = $statement->id;
                $mark['mark_group_id'] = $gropus->where('slug', $slug)->first()->id;
                if ($mark['mark_group_id'] == 10) {
                 
                    $mark['value'] = $item;
                    
                } else {
                    $mark['value'] = (int)$item;
                }
                $mark['created_at'] = now();
                $mark['updated_at'] = now();
                $save[] = $mark;
            }
            $result[] = DB::table('marks')->insert($save);
        }

        if ($statement->specialisation->faculty->id === 4) {
            if ($request->get('supervision')) {
                foreach ($request->get('supervision') as $key => $value) {
                    $supervision = App\Models\Supervision::find($value['supervision_id']);

                    if ($supervision) {
                        $supervision->update([
                            'practical_skills' => $value['practical_skills'],
                            'duty' => $value['duty'],
                            'presentation' => $value['presentation'],
                            'situational_task' => $value['situational_task'],
                            'review_of_clinics_result' => $value['review_of_clinics_result'],
                            'other' => $value['other'],
                        ]);
                    } else {
                        App\Models\Supervision::create([
                            'statement_id' => $statement->id,
                            'student_id' => $key,
                            'practical_skills' => $value['practical_skills'],
                            'duty' => $value['duty'],
                            'presentation' => $value['presentation'],
                            'situational_task' => $value['situational_task'],
                            'review_of_clinics_result' => $value['review_of_clinics_result'],
                            'other' => $value['other'],
                        ]);
                    }
                }
            }
        }

        return back();
    }

    public function print($id, $lang)
    {
        App::setLocale($lang);

        $statement = App\Models\Statement::with(['students' => function ($query) {
            $query->with(['user_attributes', 'marks'])->orderBy('last_name->ru', 'asc');
        }, 'lecturers', 'supervision'])->findOrFail($id);
        // $pdf = \PDF::setOptions([
        //     'logOutputFile' => storage_path('logs/log.htm'),
        //     'tempDir' => storage_path('logs/')
        // ])
        // ->loadView('admin.print', ['statement' => $statement]);
        // return $pdf->download('print.pdf');
        return view('admin.print', compact('statement'));
    }

    public function printExtraction($id, $lang = 'ru')
    {
        Statement::findOrFail($id);
        App::setLocale($lang);
        return MarkCrudController::studentExtractionsIndex($id);
    }
}
