<?php

namespace App\Http\Controllers\Admin;

use Backpack\Settings\app\Http\Controllers\SettingCrudController;
use Illuminate\Support\Facades\Auth;

/**
 * Class ExtendSettingCrudController
 * @package App\Http\Controllers\Admin
 * @property-read CrudPanel $crud
 */
class ExtendSettingCrudController extends SettingCrudController
{
    public function __construct()
    {
        parent::__construct();
        $this->middleware(function ($request, $next) {
            if(!empty(Auth::user()) && !Auth::user()->hasAnyRole(['SuperAdmin'])){
                return back();
            }
            return $next($request);
        });
    }

    public function setup()
    {
        parent::setup();
        $this->crud->setModel("App\Models\ExtendSetting");
        $this->crud->setColumns([[
                'name'  => 'name',
                'label' => trans('backpack::settings.name'),
            ],[
                'name'  => 'value',
                'label' => trans('backpack::settings.value'),
                'type'  => 'setting'
            ],[
                'name'  => 'description',
                'label' => trans('backpack::settings.description'),
        ]]);
    }

    public function edit($id)
    {
        $this->crud->hasAccessOrFail('update');

        $this->data['entry'] = $this->crud->getEntry($id);
        $this->crud->addField(json_decode($this->data['entry']->field, true)); // <---- this is where it's different
        $this->data['crud'] = $this->crud;
        $this->data['saveAction'] = $this->getSaveAction();
        $this->data['fields'] = $this->crud->getUpdateFields($id);
        $this->data['title'] = trans('backpack::crud.edit').' '.$this->crud->entity_name;

        $this->data['id'] = $id;

        return view($this->crud->getEditView(), $this->data);
    }
}
