<?php

namespace App\Http\Controllers\Admin;

use Backpack\PermissionManager\app\Http\Controllers\RoleCrudController;
use Auth;
/**
 * Class ExtendedRoleCrudController
 * @package App\Http\Controllers\Admin
 * @property-read CrudPanel $crud
 */
class ExtendedRoleCrudController extends RoleCrudController
{
    public function __construct()
    {
        parent::__construct();
        $this->middleware(function ($request, $next) {
            if(!empty(Auth::user()) && !Auth::user()->hasAnyRole(['SuperAdmin'])){
                return back();
            }
            return $next($request);
        });
    }

    public function setup()
    {
        $role_model = 'App\Models\ExtendedRole';
        $permission_model = 'App\Models\ExtendedPermission';

        $this->crud->setModel($role_model);
        $this->crud->setEntityNameStrings(trans('backpack::permissionmanager.role'), trans('backpack::permissionmanager.roles'));
        $this->crud->setRoute(backpack_url('roles'));

        $this->crud->addColumn([
            'name' => 'name',
            'label' => trans('base.slug'),
            'type' => 'text',
        ]);

        $this->crud->addColumn([
            'name' => 'readable_name',
            'label' => trans('base.role'),
            'type' => 'text',
        ]);

        if (config('backpack.permissionmanager.multiple_guards')) {
            $this->crud->addColumn([
                'name' => 'guard_name',
                'label' => trans('backpack::permissionmanager.guard_type'),
                'type' => 'text',
            ]);
        }

        $this->crud->addColumn([
            // n-n relationship (with pivot table)
            'label' => ucfirst(trans('backpack::permissionmanager.permission_plural')),
            'type' => 'select_multiple',
            'name' => 'permissions', // the method that defines the relationship in your Model
            'entity' => 'permissions', // the method that defines the relationship in your Model
            'attribute' => 'name', // foreign key attribute that is shown to user
            'model' => $permission_model, // foreign key model
            'pivot' => true, // on create&update, do you need to add/delete pivot table entries?
        ]);

        $this->crud->addField([
            'name' => 'name',
            'label' => trans('base.slug'),
            'type' => 'text',
            'attributes' => ['readonly' => 'readonly']
        ]);
        $this->crud->addField([
            'name' => 'readable_name',
            'label' => trans('base.role'),
            'type' => 'text',
        ]);

        if (config('backpack.permissionmanager.multiple_guards')) {
            $this->crud->addField([
                'name' => 'guard_name',
                'label' => trans('backpack::permissionmanager.guard_type'),
                'type' => 'select_from_array',
                'options' => $this->getGuardTypes(),
            ]);
        }

        $this->crud->addField([
            'label'     => ucfirst(trans('backpack::permissionmanager.permission_plural')),
            'type'      => 'checklist',
            'name'      => 'permissions',
            'entity'    => 'permissions',
            'attribute' => 'readable_name',
            'model'     => $permission_model,
            'pivot'     => true,
        ]);
    }

    private function getGuardTypes()
    {
        $guards = config('auth.guards');

        $returnable = [];
        foreach ($guards as $key => $details) {
            $returnable[$key] = $key;
        }

        return $returnable;
    }
}
