<?php

namespace App\Http\Controllers\Admin;

use App\Models\Faculty;
use Backpack\CRUD\app\Http\Controllers\CrudController;

// VALIDATION: change the requests to match your own file names if you need form validation
use App\Http\Requests\FacultyRequest as StoreRequest;
use App\Http\Requests\FacultyRequest as UpdateRequest;
use Backpack\CRUD\CrudPanel;
use Auth;

/**
 * Class FacultyCrudController
 * @package App\Http\Controllers\Admin
 * @property-read CrudPanel $crud
 */
class FacultyCrudController extends CrudController
{
    public function __construct()
    {
        parent::__construct();
        $this->middleware(function ($request, $next) {
            if(!empty(Auth::user()) && !Auth::user()->hasAnyRole(['SuperAdmin', 'Coordinator'])){
                return back();
            }
            return $next($request);
        });
    }

    public function setup()
    {
        /*
        |--------------------------------------------------------------------------
        | CrudPanel Basic Information
        |--------------------------------------------------------------------------
        */
        $this->crud->setModel('App\Models\Faculty');
        $this->crud->setRoute(config('backpack.base.route_prefix') . '/faculties');
        $this->data['names'] = ['singl', 'plurar'];

        /*
        |--------------------------------------------------------------------------
        | CrudPanel Configuration
        |--------------------------------------------------------------------------
        */
        $this->crud->setColumns([[
                'name'  => 'id',
                'label' => trans('base.id'),
            ],[
                'name'  => 'name',
                'label' => trans('base.title')
            ],[
                'name' => 'description',
                'label' => trans('base.description'),
            ],[
                'name' => 'updated_at',
                'label' => trans('base.updated'),
        ]]);

        $this->crud->addFields([[
                'name'  => 'name',
                'label' => trans('base.title'),
                'type'  => 'text'
            ],[
                'name' => 'description',
                'label' => trans('base.description'),
                'type'  => 'textarea'
            ],[
                'name' => 'updated_at',
                'label' => trans('base.updated'),
                'type'  => 'datetime'
        ]]);

        // add asterisk for fields that are required in FacultyRequest
        $this->crud->setRequiredFields(StoreRequest::class, 'create');
        $this->crud->setRequiredFields(UpdateRequest::class, 'edit');
    }

    public function store(StoreRequest $request)
    {
        // your additional operations before save here
        $redirect_location = parent::storeCrud($request);
        return $redirect_location;
    }

    public function update(UpdateRequest $request)
    {
        $redirect_location = parent::updateCrud($request);
        return $redirect_location;
    }
}
