<?php

namespace App\Http\Controllers\Admin;

use App\Http\Controllers\MoodleController;
use App\Models\Group;
use Backpack\CRUD\app\Http\Controllers\CrudController;

// VALIDATION: change the requests to match your own file names if you need form validation
use App\Http\Requests\GroupRequest as StoreRequest;
use App\Http\Requests\GroupRequest as UpdateRequest;
use Backpack\CRUD\CrudPanel;
use Auth;
use Illuminate\Database\Query\Builder;
use Illuminate\Http\Request;
use Illuminate\Support\Arr;

/**
 * Class GroupCrudController
 * @package App\Http\Controllers\Admin
 * @property-read CrudPanel $crud
 */
class GroupCrudController extends CrudController
{
    public function __construct()
    {
        parent::__construct();
        $this->middleware(function ($request, $next) {
            if(!empty(Auth::user()) && !Auth::user()->hasAnyRole(['Coordinator', 'SuperAdmin'])){
                return back();
            }
            return $next($request);
        });
    }

    public function setup()
    {
        /*
        |--------------------------------------------------------------------------
        | CrudPanel Basic Information
        |--------------------------------------------------------------------------
        */
        $this->crud->setModel('App\Models\Group');
        $this->crud->setRoute(config('backpack.base.route_prefix') . '/groups');
        $this->crud->setEntityNameStrings('group', 'groups');

        /*
        |--------------------------------------------------------------------------
        | CrudPanel Configuration
        |--------------------------------------------------------------------------
        */

         // TODO добавить фильтр по факультетут

        $this->crud->setColumns([[
                'name'  => 'id',
                'label' => trans('base.id'),
            ],[
                'name'  => 'specialisation_id',
                'label' => trans('base.specialisation'),
                'type' => 'select',
                'entity' => 'specialisation',
                'attribute' => 'name',
                'model' => "App\Models\Specialisation"
            ],[
                'name'  => 'name',
                'label' => trans('base.title')
            ],[
                'name' => 'description',
                'label' => trans('base.description'),
        ],[
            'name' => 'status',
            'label' => trans('base.description'),
        ]]);

        $this->crud->addFields([[
                'name'  => 'specialisation_id',
                'label' => trans('base.specialisation'),
                'type'  => 'select2', // TODO добавить ведомое поле факультета
                'entity' => 'specialisation',
                'attribute' => 'name',
                'model' => "App\Models\Specialisation"
            ],[
                'name'  => 'name',
                'label' => trans('base.title'),
                'type'  => 'text'
            ],[
                'name' => 'description',
                'label' => trans('base.description'),
                'type'  => 'textarea'
        ]]);

        $this->crud->addField([
            'name' => 'program_slug',
            'label' => __('base.group.program'),
            'type'  => 'select2_from_array',
            'wrapperAttributes' => [
                'class' => 'form-group col-md-4'
            ],
            'options' => Arr::pluck(json_decode(config('settings.studyPrograms'), true), 'name', 'slug'),
        ]);

        $this->crud->addField([
            'name' => 'stage',
            'label' => __('base.group.stage'),
            'type' => 'select2_from_array',
            'wrapperAttributes' => [
                'class' => 'form-group col-md-4'
            ],
            'options' => Arr::pluck(json_decode(config('settings.stage'), true), 'name', 'slug'),
        ]);

        $this->crud->addField([
            'name' => 'semester',
            'label' => __('base.group.semester'),
            'type' => 'select2_from_array',
            'wrapperAttributes' => [
                'class' => 'form-group col-md-4'
            ],
            'options' => Arr::pluck(json_decode(config('settings.semester'), true), 'name', 'slug'),
        ]);

        // add asterisk for fields that are required in GroupRequest
        $this->crud->setRequiredFields(StoreRequest::class, 'create');
        $this->crud->setRequiredFields(UpdateRequest::class, 'edit');
    }

    public function store(StoreRequest $request)
    {
        // your additional operations before save here
        $redirect_location = parent::storeCrud($request);

        MoodleController::createCohort(
            $this->crud->entry->id,
            $request->get('name')
        );
        // your additional operations after save here
        // use $this->data['entry'] or $this->crud->entry
        return $redirect_location;
    }

    public function update(UpdateRequest $request)
    {
        // your additional operations before save here
        $redirect_location = parent::updateCrud($request);
        // your additional operations after save here
        // use $this->data['entry'] or $this->crud->entry
        return $redirect_location;
    }

    public function getAjaxGroups(Request $request)
    {
        $group = $request->get('group');
        $specialisation = $request->input('specialisation');
//        if( !empty($request->input('specialisation')) ){
//            $options = Group::whereHas('specialisation', function(Builder $query) use ($specialisation) {
//                $query->where('id', $specialisation );
//            })
//                ->where('name', 'like', '%'.$group.'%')->get()->pluck('name', 'id')->toArray();
//            return $options;
//        } else {
//            $options = Group::where('name', 'like', '%'.$group.'%')->get()->pluck('name', 'id')->toArray();
//            return $options;
//        }
        //dd($group); // FR 11/07
        return Group::where('name', 'like' , "%$group%")->get()->pluck('name', 'id')->toArray();

    }
}
