<?php

namespace App\Http\Controllers\Admin;

use Backpack\CRUD\app\Http\Controllers\CrudController;
use App;
use Auth;
use App\Models\Log;
use Backpack\PermissionManager\app\Models\Role;
use Illuminate\Database\Eloquent\Builder;


class LogCrudController extends CrudController
{
    public function __construct()
    {
        parent::__construct();
        $this->middleware(function ($request, $next) {
            if (!empty(Auth::user()) && !Auth::user()->hasAnyRole(['SuperAdmin'])) {
                return back();
            }
            return $next($request);
        });

    }

    public function setup()
    {
        App::setLocale(session('language', 'ka'));
        /*
        |--------------------------------------------------------------------------
        | CrudPanel Basic Information
        |-------------------------------------------------------------------------
        */

        $this->crud->setModel('App\Models\Log');
        $this->crud->setRoute(config('backpack.base.route_prefix') . '/log_users');
        $this->crud->setEntityNameStrings(__('base.statements_add.single'), __('base.logs_users.plural'));
        /*
        |--------------------------------------------------------------------------
        | CrudPanel Configuration
        |--------------------------------------------------------------------------
        */

        $this->crud->addColumn([
            'name' => 'user_full_name',
            'label' => __('base.logs_users.full_name'),
            'type' => 'string',
        ]);
        $this->crud->addColumn([
            'name' => 'user_role',
            'label' => __('base.logs_users.role'),
            'type' => 'string',
        ]);

        $this->crud->addColumn([
            'name' => 'types',
            'label' => __('base.logs_users.type'),
            'type' => 'string',
        ]);

        $this->crud->addColumn([
            'name' => 'statement_id',
            'label' => __('base.logs_users.statement'),
            'type' => 'string',
        ]);

        $this->crud->addColumn([
            'name' => 'created_at',
            'label' => __('base.logs_users.date'),
            'type' => 'date',
        ]);


        $this->crud->addFilter([
            'type' => 'date_range',
            'name' => 'created_at',
            'label' => __('base.logs_users.date')
        ],
            false,
            function ($value) { // if the filter is active, apply these constraints
                $dates = json_decode($value);
                $this->crud->addClause('where', 'created_at', '>=', $dates->from);
                $this->crud->addClause('where', 'created_at', '<=', $dates->to . ' 23:59:59');
            });

        $this->crud->addFilter([
            'type' => 'text',
            'name' => 'user_full_name',
            'label' => __('base.logs_users.full_name')
        ], null, function ($value) {
            $this->crud->addClause('where', 'user_full_name', 'LIKE', "%$value%");
        });

        $this->crud->addFilter([
            'type' => 'dropdown',
            'name' => 'user_role',
            'label' => __('base.logs_users.role')
        ],
            [
                'Coordinator' => 'Coordinator',
                'Lecturer' => 'Lecturer'
            ], function ($value) {
                $this->crud->addClause('where', 'user_role', '=', $value);
            });


        $dropdownType = Log::AVAILABLE_TYPE;

        foreach ($dropdownType as $key => $value) {
            $dropdownType[$key] = __('base.logs_users.' . $key);
        }

        $this->crud->addFilter([
            'type' => 'dropdown',
            'name' => 'type',
            'label' => __('base.logs_users.type')
        ], $dropdownType, function ($value) {
            $name = Log::AVAILABLE_TYPE[$value];
            if (isset($name)) {
                $this->crud->addClause('where', 'type', '=', $name);
            }
        });
        
        $this->crud->addFilter([
            'type' => 'text',
            'name' => 'statement_id',
            'label' => __('base.logs_users.statement')
        ], null, function ($value) {
            $this->crud->addClause('where', 'statement_id', '=', $value);
            
        });


        $this->crud->removeButton('update');
        $this->crud->removeButton('create');
    }
}
