<?php

namespace App\Http\Controllers\Admin;

use App\Models\Mark;
use App\Models\Student;
use Backpack\CRUD\app\Http\Controllers\CrudController;

// VALIDATION: change the requests to match your own file names if you need form validation
use App\Http\Requests\MarkRequest as StoreRequest;
use App\Http\Requests\MarkRequest as UpdateRequest;
use Backpack\CRUD\CrudPanel;
use Carbon\Carbon;
use Illuminate\Database\Eloquent\Builder;
use Illuminate\Support\Arr;
use \DB;
use App\Models\Statement;
use Illuminate\Support\Facades\Auth;
use function foo\func;

/**
 * Class MarkCrudController
 * @package App\Http\Controllers\Admin
 * @property-read CrudPanel $crud
 */
class MarkCrudController extends CrudController
{
    public function __construct()
    {
        parent::__construct();
        $this->middleware(function ($request, $next) {
            if(!empty(Auth::user()) && !Auth::user()->hasAnyRole(['SuperAdmin', 'Coordinator', 'Lecturer', 'Student'])){
                return back();
            }
            return $next($request);
        });
    }

    public function setup()
    {
        /*
        |--------------------------------------------------------------------------
        | CrudPanel Basic Information
        |--------------------------------------------------------------------------
        */
        //$this->crud->setModel('App\Models\Mark');
        $this->crud->setRoute(config('backpack.base.route_prefix') . '/marks');
        $this->crud->setEntityNameStrings('mark', 'marks');

        /*
        |--------------------------------------------------------------------------
        | CrudPanel Configuration
        |--------------------------------------------------------------------------
        */

        // // add asterisk for fields that are required in MarkRequest
        // $this->crud->setRequiredFields(StoreRequest::class, 'create');
        // $this->crud->setRequiredFields(UpdateRequest::class, 'edit');
    }

    public function index()
    {
        $statements = Statement::whereHas('students', function($query){
            $query->where('id', Auth::user()->id);
        });
        if( !empty($semester = request()->get('semester')) ){
            $statements->where('semester', $semester);
        }
        $statements = $statements
            ->paginate(30);
        $statements_gpa = Statement::whereHas('students', function($q){
            $q->where('id', Auth::user()->id);
        })->when(!empty($semester = request()->get('semester')), function ($q) use ($semester) {
            return $q->where('semester', $semester);
        })->get();
        $gpa = $this->gpa( Auth::user()->id, $statements_gpa );

        $semesters = Arr::pluck(json_decode(config('settings.semester'), true), 'name', 'slug');
        return view('admin.student-marks', compact('statements', 'semesters', 'gpa'));
    }

    public function studentSubjectList($id)
    {
        $statements = Statement::whereHas('students', function($query) use ($id){
            $query->where('id', $id);
        });
        if( !empty($semester = request()->get('semester')) ){
            $statements= $statements->where('semester', $semester);
        }
        $statements = $statements->get();

        $gpa = $this->gpa( $id, $statements );
        $status = Student::find(\Auth::id());
        if($status) $status=$status->user_attributes()->where('key', 'status')->value('value');
        $semesters = Arr::pluck(json_decode(config('settings.semester'), true), 'name', 'slug');
        return view('admin.marks', compact('statements', 'semesters', 'gpa','status'));
    }

    private function gpa($student_id, $statements)
    {
        $statement_ids = [];
        foreach ($statements as $statement) {
            $statement_ids[] = $statement->id;
        }
        $gpa = Mark::where('user_id', $student_id)
                ->whereIn('statement_id', $statement_ids)
                ->where('mark_group_id', 9)
                ->where('value', '>', 0)
                ->avg('value');
        return Mark::toECTS($gpa);
    }

    public static function studentExtractionsIndex($id)
    {
        $student = Student::where('id', $id)->first();
        $student_credentials = $student->full_name;
        $marks = Mark::where('user_id', $id)
            ->where('mark_group_id', 9)
            ->with('statement.subject')
            ->get();
       
        $marks = $marks->sortBy('value');
        
        
         foreach($marks as $key =>  $mark){
            foreach($marks as $key2 => $mark2){
                if($mark->subject_id == $mark2->subject_id){
        
                    if(strtotime($mark->created_at) != strtotime($mark2->created_at))
                    if(strtotime($mark->created_at) > strtotime($mark2->created_at)){
                        unset($marks[$key2]);
                    } elseif(strtotime($mark->created_at) < strtotime($mark2->created_at)) {
                        unset($marks[$key]);
                    }
                }
            }
        }
        
        $marks = $marks->unique(function ($item){
            return $item->statement->subject->name;
        });
        $unique_marks = $marks;
        
        
      
        $unique_marks = $marks->unique(function ($item){
            return $item->statement->id;
        });
        
        $total_count = 0;
        $total_ects = 0;
        $total_gpa = 0;
        $total_credits = 0;
        $iteration = 0;


       


        foreach ($unique_marks as $entry){
           $marsk = $student->marks->where('statement_id', $entry->statement_id)->pluck('value', 'mark_group.slug');

            if($marsk['total'] != null)
                $entry->value = $marsk['total'];
           
            //var_dump($marsk);
            if (!empty($entry->value) && $entry->value >= 51 ){


                $final_exam = Mark::where('statement_id', $entry->statement_id)->where('user_id', $entry->user_id)->where('mark_group_id', 7)->first();
                $additional_exam = Mark::where('statement_id', $entry->statement_id)->where('user_id', $entry->user_id)->where('mark_group_id', 8)->first();

                if ($final_exam->value < 20){
                    $entry->fucking_variable = 'Fx ' . __('base.extractions.can_pass_additional_exam');
                } elseif ($final_exam->value < 20 && $additional_exam->value <20){
                    $entry->fucking_variable = 'F ' . __('base.extractions.dont_pass');
                } else {
                    if( 91 <= $entry->value){
                        $entry->fucking_variable = 'A ';
                    } elseif( 81 <= $entry->value && $entry->value <= 90){
                        $entry->fucking_variable =$entry->fucking_variable ='B';
                    } elseif( 71 <= $entry->value && $entry->value <=80 ){
                        $entry->fucking_variable = 'C ';
                    } elseif( 61 <= $entry->value && $entry->value <= 70 ){
                        $entry->fucking_variable = 'D ';
                    } elseif( 51 <= $entry->value && $entry->value <= 60 ) {
                        $entry->fucking_variable = 'E ';
                    }
//                    elseif( $entry->value == 0 ) {
//                        $entry->fucking_variable = 'NO VALUE';
//                    }
                    $total_gpa += $entry->gpa;
                    $total_ects += $entry->ects;
                    $total_credits += $entry->credits;
                }


                $iteration += 1;
            }
        }
        
        if ($total_credits != 0){
            $resulting_gpa = sprintf('%0.2f', $total_ects / $total_credits);
        } else {
            $resulting_gpa = 'No credits';
        }
        
        $reorder = [];
        foreach($marks as $mark){
            $reorder[$mark->statement->semestername][] = $mark;
        }
        ksort($reorder);
        $marks = $reorder;

        return view('admin.print_extractions', compact('marks', 'resulting_gpa', 'total_ects', 'student_credentials'));
    }

//    public function studentSubjectList($id)
//    {
//        $statements = Statement::whereHas('students', function($query){
//            $query->where('id', request()->segment(2));
//        });
//        if( !empty($semester = request()->get('semester')) ){
//            $statements->where('semester', $semester);
//        }
//        $statements = $statements->get();
//        $gpa = $this->gpa( $id, $statements );
//
//        $semesters = Arr::pluck(json_decode(config('settings.semester'), true), 'name', 'slug');
//        return view('admin.marks', compact('statements', 'semesters', 'gpa'));
//    }

    // public function store(StoreRequest $request)
    // {
    //     // your additional operations before save here
    //     $redirect_location = parent::storeCrud($request);
    //     // your additional operations after save here
    //     // use $this->data['entry'] or $this->crud->entry
    //     return $redirect_location;
    // }

    // public function update(UpdateRequest $request)
    // {
    //     // your additional operations before save here
    //     $redirect_location = parent::updateCrud($request);
    //     // your additional operations after save here
    //     // use $this->data['entry'] or $this->crud->entry
    //     return $redirect_location;
    // }
}
