<?php

namespace App\Http\Controllers\Admin;

use Backpack\CRUD\app\Http\Controllers\CrudController;

// VALIDATION: change the requests to match your own file names if you need form validation
use App\Http\Requests\SyllabusRequest as StoreRequest;
use App\Http\Requests\SyllabusRequest as UpdateRequest;
use Backpack\CRUD\CrudPanel;
use Illuminate\Support\Arr;

/**
 * Class SyllabusCrudController
 * @package App\Http\Controllers\Admin
 * @property-read CrudPanel $crud
 */
class ProgramDocumentCrudController extends CrudController
{
    public function setup()
    {
        /*
        |--------------------------------------------------------------------------
        | CrudPanel Basic Information
        |--------------------------------------------------------------------------
        */
        $this->crud->setModel('App\Models\ProgramDocument');
        $this->crud->setRoute(config('backpack.base.route_prefix') . '/programs');
        $this->crud->setEntityNameStrings(__('base.program_table.single'), __('base.program_table.plural'));

        /*
        |--------------------------------------------------------------------------
        | CrudPanel Configuration
        |--------------------------------------------------------------------------
        */

        // TODO: remove setFromDb() and manually define Fields and Columns
        $this->crud->setFromDb();

        $this->crud->allowAccess('create');

        // add asterisk for fields that are required in SyllabusRequest
        $this->crud->setRequiredFields(StoreRequest::class, 'create');
        $this->crud->setRequiredFields(UpdateRequest::class, 'edit');

        $this->crud->setColumns([
            [
                'name'  => 'name',
                'label' => trans('base.syllabus.name'),
                'type' => "text",
                'function_name' => 'getCardId',
            ],
            [
                'name'  => 'category',
                'label' => __('base.syllabus.category'),
                'type' => "model_function",
                'function_name' => 'getCategory',
            ],
            [
                'name'  => 'faculty',
                'label' => __('base.faculty'),
                'type' => "model_function",
                'function_name' => 'getFacultyName',
            ],
        ]);

        $this->crud->addField([
            'name'  => 'name',
            'label' => __('base.syllabus.name'),
            'type'  => 'text'
        ]);

        if(!empty( config('settings.syllabus_categories') )){
            $this->crud->addField([
                'name' => 'category',
                'label' => __('base.syllabus.category'),
                'type' => 'select2_from_array',
                'options' => Arr::pluck(json_decode(config('settings.studyPrograms'), true), 'name', 'slug'),
                'wrapperAttributes' => [
                    'class' => 'form-group col-md-6'
                ]
            ]);
        }

        $this->crud->addField([
            'name' => 'faculty_id',
            'label' => __('base.faculty'),
            'type' => 'select2',
            'entity' => 'faculty',
            'attribute' => 'name',
            'model' => "App\Models\Faculty",
            'wrapperAttributes' => [
                'class' => 'form-group col-md-6'
            ]
        ]);

        $this->crud->addField([
            'name' => 'file',
            'label' => __('base.syllabus.file'),
            'type' => 'upload',
            'upload' => true,
            'disk' => 'program_document'
        ]);
    }

    public function store(StoreRequest $request)
    {
        // your additional operations before save here
        $redirect_location = parent::storeCrud($request);
        // your additional operations after save here
        // use $this->data['entry'] or $this->crud->entry
        return $redirect_location;
    }

    public function update(UpdateRequest $request)
    {
        // your additional operations before save here
        $redirect_location = parent::updateCrud($request);
        // your additional operations after save here
        // use $this->data['entry'] or $this->crud->entry
        return $redirect_location;
    }
}
