<?php

namespace App\Http\Controllers\Admin;

use App\Models\Questionnaire;
use Backpack\CRUD\app\Http\Controllers\CrudController;

// VALIDATION: change the requests to match your own file names if you need form validation
use App\Http\Requests\QuestionnaireRequest as StoreRequest;
use App\Http\Requests\QuestionnaireRequest as UpdateRequest;
use Backpack\CRUD\CrudPanel;
use Illuminate\Support\Arr;

/**
 * Class QuestionnaireCrudController
 * @package App\Http\Controllers\Admin
 * @property-read CrudPanel $crud
 */
class QuestionnaireCrudController extends CrudController
{
    public $questionnaire = null;

    public function setup()
    {
        /*
        |--------------------------------------------------------------------------
        | CrudPanel Basic Information
        |--------------------------------------------------------------------------
        */
        $this->crud->setModel('App\Models\Questionnaire');
        $this->crud->setRoute(config('backpack.base.route_prefix') . '/questionnaire');
        $this->crud->setEntityNameStrings( __('base.questionnaire_add.single'), __('base.questionnaire_add.plural') );
        $this->crud->allowAccess('clone');
        if(!(request()->segment(2) == 'create')){
            $this->questionnaire = Questionnaire::find(request()->segment(2));
        };

        /*
        |--------------------------------------------------------------------------
        | CrudPanel Configuration
        |--------------------------------------------------------------------------
        */
        $this->crud->setColumns([
            [
                'name'  => 'href',
                'label' => trans('base.questionnaire.link'),
                'type' => "link",
            ],[
                'name'  => 'description',
                'label' => trans('base.questionnaire.description'),
                'type' => "text",
            ],[
                'name'  => 'status',
                'label' => trans('base.questionnaire.status'),
                'type' => "check",
            ],[
                'name'  => 'faculty.name',
                'label' => trans('base.faculty'),
                'type' => "text",
            ],[
                'name'  => 'semester',
                'label' => trans('base.questionnaire.semesters'),
                'type' => "array",
            ],[
                'name'  => 'expires_on',
                'label' => trans('base.questionnaire.expires_on'),
                'type' => "date",
            ]
        ]);

        $this->crud->addField([
            'name' => 'href',
            'label' => __('base.questionnaire.link'),
            'type' => 'url',
        ]);

        $this->crud->addField([
            'name' => 'description',
            'label' => __('base.questionnaire.description'),
            'type' => 'textarea'
        ]);

        $this->crud->addField([  // Select2
            'label' => __('base.faculty'),
            'type' => 'select2',
            'name' => 'faculty_id', // the db column for the foreign key
            'entity' => 'faculty', // the method that defines the relationship in your Model
            'attribute' => 'name', // foreign key attribute that is shown to user
            'model' => "App\Models\Faculty", // foreign key model

            // optional
        ]);

//        dd(json_decode($this->questionnaire->semester));
        $this->crud->addField([
            'name' => 'semester',
            'label' => __('base.questionnaire.semester'),
            'type' => 'select2_from_array',
            'options' => Arr::pluck(json_decode(config('settings.semester'), true), 'name', 'slug'),
            'allows_multiple' => true,
            'value' => (!empty($this->questionnaire) ? json_decode($this->questionnaire->semester) : ''),
            'fake' => true,
        ]);

        $this->crud->addField([
            'name' => 'status',
            'label' => __('base.questionnaire.status'),
            'type' => 'checkbox',
            'wrapperAttributes' => [
                'class' => 'form-group col-md-4'
            ],
        ]);

        $this->crud->addField([
            'name' => 'expires_on',
            'label' => __('base.questionnaire.expires_on'),
            'type' => 'date',
            'allows_null' => true,
            'wrapperAttributes' => [
                'class' => 'form-group col-md-4'
            ],
        ]);

        // add asterisk for fields that are required in QuestionnaireRequest
        $this->crud->setRequiredFields(StoreRequest::class, 'create');
        $this->crud->setRequiredFields(UpdateRequest::class, 'edit');
    }

    public function store(StoreRequest $request)
    {
        $entry = $this->crud->entry;
//        dd($request);
        // your additional operations before save here
        $redirect_location = parent::storeCrud($request);
        $this->crud->entry->semester = json_encode($request->get('semester'));
        $this->crud->entry->save();
        // your additional operations after save here
        // use $this->data['entry'] or $this->crud->entry
        return $redirect_location;
    }

    public function update(UpdateRequest $request)
    {
        // your additional operations before save here
        $redirect_location = parent::updateCrud($request);
        $this->questionnaire->semester = json_encode($request->get('semester'));
        $this->questionnaire->save();
        // your additional operations after save here
        // use $this->data['entry'] or $this->crud->entry
        return $redirect_location;
    }
}
