<?php

namespace App\Http\Controllers\Admin;

use App\Models\Specialisation;
use Backpack\CRUD\app\Http\Controllers\CrudController;

// VALIDATION: change the requests to match your own file names if you need form validation
use App\Http\Requests\SpecialisationRequest as StoreRequest;
use App\Http\Requests\SpecialisationRequest as UpdateRequest;
use Backpack\CRUD\CrudPanel;
use Auth;
use Illuminate\Http\Request;

/**
 * Class SpecialisationCrudController
 * @package App\Http\Controllers\Admin
 * @property-read CrudPanel $crud
 */
class SpecialisationCrudController extends CrudController
{
    public function __construct()
    {
        parent::__construct();
        $this->middleware(function ($request, $next) {
            if(!empty(Auth::user()) && !Auth::user()->hasAnyRole(['SuperAdmin'])){
                return back();
            }
            return $next($request);
        });
    }

    public function setup()
    {
        /*
        |--------------------------------------------------------------------------
        | CrudPanel Basic Information
        |--------------------------------------------------------------------------
        */
        $this->crud->setModel('App\Models\Specialisation');
        $this->crud->setRoute(config('backpack.base.route_prefix') . '/specialisations');
        $this->crud->setEntityNameStrings('specialisation', 'specialisations');

        /*
        |--------------------------------------------------------------------------
        | CrudPanel Configuration
        |--------------------------------------------------------------------------
        */
        $this->crud->setColumns([[
                'name'  => 'id',
                'label' => trans('base.id'),
            ],[
                'name'  => 'faculty_id',
                'label' => trans('base.faculty'),
                'type'  => 'select',
                'entity' => 'faculty',
                'attribute' => 'name',
                'model' => "App\Models\Faculty"
            ],[
                'name'  => 'name',
                'label' => trans('base.title')
            ],[
                'name' => 'description',
                'label' => trans('base.description'),
            ]]);

        $this->crud->addFields([[
                'name'  => 'faculty_id',
                'label' => trans('base.faculty'),
                'type'  => 'select2',
                'entity' => 'faculty',
                'attribute' => 'name',
                'model' => "App\Models\Faculty"
            ],[
                'name'  => 'name',
                'label' => trans('base.title'),
                'type'  => 'text'
            ],[
                'name' => 'description',
                'label' => trans('base.description'),
                'type'  => 'textarea'
        ]]);

        // TODO: remove setFromDb() and manually define Fields and Columns
        // $this->crud->setFromDb();

        // add asterisk for fields that are required in SpecialisationRequest
        $this->crud->setRequiredFields(StoreRequest::class, 'create');
        $this->crud->setRequiredFields(UpdateRequest::class, 'edit');
    }

    public function store(StoreRequest $request)
    {
        // your additional operations before save here
        $redirect_location = parent::storeCrud($request);
        // your additional operations after save here
        // use $this->data['entry'] or $this->crud->entry
        return $redirect_location;
    }

    public function update(UpdateRequest $request)
    {
        // your additional operations before save here
        $redirect_location = parent::updateCrud($request);
        // your additional operations after save here
        // use $this->data['entry'] or $this->crud->entry
        return $redirect_location;
    }
    public function getAjaxSpecialisations(Request $request)
    {
        $faculty = $request->get('faculty');
        $specialisation = $request->get('specialisation');
        if( !empty($request->input('faculty')) ){
            $options = Specialisation::whereHas('specialisation', function(Builder $query) use ($faculty) {
                $query->where('id', $faculty );
            })
                ->where('name', 'like', '%'.$specialisation.'%')->get()->pluck('name', 'id')->toArray();
            return $options;
        } else {
            $options = Specialisation::where('name', 'like', '%'.$specialisation.'%')->get()->pluck('name', 'id')->toArray();
            return $options;
        }
        //dd($group); // FR 11/07
        return Group::where('name', 'like' ,"%$group%")->get()->pluck('name', 'id')->toArray();

    }
}
