<?php

namespace App\Http\Controllers\Admin;

use Backpack\CRUD\app\Http\Controllers\CrudController;

// VALIDATION: change the requests to match your own file names if you need form validation
use App\Http\Requests\SpreadsheetRequest as StoreRequest;
use App\Http\Requests\SpreadsheetRequest as UpdateRequest;
use Backpack\CRUD\CrudPanel;

/**
 * Class SpreadsheetCrudController
 * @package App\Http\Controllers\Admin
 * @property-read CrudPanel $crud
 */
class SpreadsheetCrudController extends CrudController
{
    public function setup()
    {
        /*
        |--------------------------------------------------------------------------
        | CrudPanel Basic Information
        |--------------------------------------------------------------------------
        */
        $this->crud->setModel('App\Models\Spreadsheet');
        $this->crud->setRoute(config('backpack.base.route_prefix') . '/spreadsheet');

        $this->crud->setEditView('admin.custom.edit');
        $this->crud->setCreateView('admin.custom.create');

        $this->crud->setEntityNameStrings(__('base.spreadsheets.single_name'), __('base.spreadsheets.plural_name'));

        /*
        |--------------------------------------------------------------------------
        | CrudPanel Configuration
        |--------------------------------------------------------------------------
        */

        $this->crud->addColumn([
            'name' => 'name', // The db column name
            'label' => __('base.title')
        ]);

        $this->crud->addField([
            'name' => 'name',
            'type' => 'text',
            'label' => __('base.title')
        ]);

        $this->crud->addField([
            'name' => 'content',
            'type' => 'spreadsheet'
        ]);

        // add asterisk for fields that are required in SpreadsheetRequest
        $this->crud->setRequiredFields(StoreRequest::class, 'create');
        $this->crud->setRequiredFields(UpdateRequest::class, 'edit');
    }

    public function store(StoreRequest $request)
    {
        // your additional operations before save here
        $redirect_location = parent::storeCrud($request);
        // your additional operations after save here
        // use $this->data['entry'] or $this->crud->entry
        return $redirect_location;
    }

    public function update(UpdateRequest $request)
    {
        // your additional operations before save here
        $redirect_location = parent::updateCrud($request);
        // your additional operations after save here
        // use $this->data['entry'] or $this->crud->entry
        return $redirect_location;
    }
}
