<?php

namespace App\Http\Controllers\Admin;

use App\Models\Faculty;
use Backpack\CRUD\app\Http\Controllers\CrudController;

// VALIDATION: change the requests to match your own file names if you need form validation
use App\Http\Requests\SubjectRequest as StoreRequest;
use App\Http\Requests\SubjectRequest as UpdateRequest;
use Backpack\CRUD\CrudPanel;
use Auth;
use Illuminate\Database\Eloquent\Builder;
use Illuminate\Support\Arr;

/**
 * Class SubjectCrudController
 * @package App\Http\Controllers\Admin
 * @property-read CrudPanel $crud
 */
class SubjectCrudController extends CrudController
{
    public function __construct()
    {
        parent::__construct();
        $this->middleware(function ($request, $next) {
            if(!empty(Auth::user()) && !Auth::user()->hasAnyRole(['Coordinator', 'SuperAdmin'])){
                return back();
            }
            return $next($request);
        });
    }

    public function setup()
    {
        /*
        |--------------------------------------------------------------------------
        | CrudPanel Basic Information
        |--------------------------------------------------------------------------
        */
        $this->crud->setModel('App\Models\Subject');
        $this->crud->setRoute(config('backpack.base.route_prefix') . '/subjects');
        $this->crud->setEntityNameStrings(__('base.subjects_single'), __('base.subjects_plural'));

        /*
        |--------------------------------------------------------------------------
        | CrudPanel Configuration
        |--------------------------------------------------------------------------
        */

        $this->crud->setColumns([[
                'name'  => 'id',
                'label' => trans('base.id'),
            ],[
                'name'  => 'faculties',
                'label' => trans('base.faculties'),
                'type' => 'select_multiple',
                'entity' => 'faculties',
                'attribute' => 'name',
                'model' => "App\Models\Faculty"
            ],[
                'name'  => 'name',
                'label' => trans('base.title')
            ],[
                'name' => 'description',
                'label' => trans('base.description'),
        ]]);

        $this->crud->addFields([[
                'name'  => 'faculties',
                'label' => trans('base.faculties'),
                'type'  => 'select2_multiple', // TODO добавить ведомое поле факультета
                'entity' => 'faculties',
                'attribute' => 'name',
                'model' => "App\Models\Faculty",
                'pivot' => true,
            ],[
                'name'  => 'name',
                'label' => trans('base.title'),
                'type'  => 'text'
            ],[
                'name' => 'description',
                'label' => trans('base.description'),
                'type'  => 'textarea'
        ]]);
        $this->crud->addFilter([
            'name' => 'name',
            'type' => 'text',
            'label' => __('base.title')
        ],
            false,
            function ($value) {
                $this->crud->query = $this->crud->query->where('name', 'like', "%$value%");
            });

        $this->crud->addFilter([
            'name' => 'faculties',
            'type' => 'select2',
            'label'=> __('base.faculty')
        ],
            Faculty::all()->push(['name'=>'Без факултета','id'=>'not'])->pluck('name', 'id')->toArray(),
            function($values) {
                if($values=='not')
                $this->crud->query = $this->crud->query->doesnthave('faculties');
                else
                $this->crud->query = $this->crud->query->has('faculties')->whereHas('faculties', function (Builder $query) use ($values) {
                     $query->where('id',  $values);
                });
            });

        // TODO: remove setFromDb() and manually define Fields and Columns
        // $this->crud->setFromDb();

        // add asterisk for fields that are required in SubjectRequest
        $this->crud->setRequiredFields(StoreRequest::class, 'create');
        $this->crud->setRequiredFields(UpdateRequest::class, 'edit');
    }

    public function store(StoreRequest $request)
    {
        // your additional operations before save here
        $redirect_location = parent::storeCrud($request);
        // your additional operations after save here
        // use $this->data['entry'] or $this->crud->entry
        return $redirect_location;
    }

    public function update(UpdateRequest $request)
    {
        // your additional operations before save here
        $redirect_location = parent::updateCrud($request);
        // your additional operations after save here
        // use $this->data['entry'] or $this->crud->entry
        return $redirect_location;
    }
}
