<?php

namespace App\Http\Controllers\Admin;

use Backpack\CRUD\app\Http\Controllers\CrudController;

// VALIDATION: change the requests to match your own file names if you need form validation
use App\Http\Requests\SyllabusRequest as StoreRequest;
use App\Http\Requests\SyllabusRequest as UpdateRequest;
use Backpack\CRUD\CrudPanel;
use Illuminate\Support\Arr;
use App\Models\Syllabus;
use App\Models\Faculty;

/**
 * Class SyllabusCrudController
 * @package App\Http\Controllers\Admin
 * @property-read CrudPanel $crud
 */
class SyllabusCrudController extends CrudController
{
    public function setup()
    {
        /*
        |--------------------------------------------------------------------------
        | CrudPanel Basic Information
        |--------------------------------------------------------------------------
        */
        $this->crud->setModel('App\Models\Syllabus');
        $this->crud->setRoute(config('backpack.base.route_prefix') . '/syllabus');
        $this->crud->setEntityNameStrings(__('base.syllabus.single'), __('base.syllabus.plural'));

        /*
        |--------------------------------------------------------------------------
        | CrudPanel Configuration
        |--------------------------------------------------------------------------
        */

        // TODO: remove setFromDb() and manually define Fields and Columns
        $this->crud->setFromDb();

        // add asterisk for fields that are required in SyllabusRequest
        $this->crud->setRequiredFields(StoreRequest::class, 'create');
        $this->crud->setRequiredFields(UpdateRequest::class, 'edit');

        $this->crud->setColumns([
            [
                'name'  => 'name',
                'label' => trans('base.syllabus.name'),
                'type' => "text",
                'function_name' => 'getCardId',
            ],
            [
                'name'  => 'category',
                'label' => __('base.syllabus.category'),
                'type' => "model_function",
                'function_name' => 'getSyllabusCategory',
            ],
            [
                'name'  => 'faculty',
                'label' => __('base.faculty'),
                'type' => "model_function",
                'function_name' => 'getFacultyName',
            ],
        ]);

        $this->crud->addField([
            'name'  => 'name',
            'label' => __('base.syllabus.name'),
            'type'  => 'text'
        ]);
        $facultets = Faculty::all();
        $facults = [];
        foreach($facultets as $faculty){
            $facults[$faculty->id] = $faculty->name;
        }
        
        $categoriesOld = json_decode(config('settings.syllabus_categories')); 
        $categories = [];
        foreach($categoriesOld as $category){
            $categories[$category->slug] = $category->name;
        }

        if(!empty( config('settings.syllabus_categories') )){
            $this->crud->addField([
                'name' => 'category',
                'label' => __('base.syllabus.category'),
                'type' => 'select2_from_array',
                'options' => Arr::pluck(json_decode(config('settings.syllabus_categories'), true), 'name', 'slug'),
                'wrapperAttributes' => [
                    'class' => 'form-group col-md-6'
                ]
            ]);
        }

        $this->crud->addField([
            'name' => 'faculty_id',
            'label' => __('base.faculty'),
            'type' => 'select2',
            'entity' => 'faculty',
            'attribute' => 'name',
            'model' => "App\Models\Faculty",
            'wrapperAttributes' => [
                'class' => 'form-group col-md-6'
            ]
        ]);
        $this->crud->addField([
            'name' => 'file',
            'label' => __('base.syllabus.file'),
            'type' => 'upload',
            'upload' => true,
            'disk' => 'syllabus'
        ]);
        
        
         $this->crud->addFilter([
            'type' => 'text',
            'name' => 'name',
            'label' => trans('base.syllabus.name')
        ],
            false,
            function ($value) { // if the filter is active
                $this->crud->addClause('where', 'name', 'LIKE', "%$value%");
            });
            
        $this->crud->addFilter([
            'type' => 'select2',
            'name' => 'category',
            'label' => trans('base.syllabus.category')
        ],
            $categories,
            function ($value) { // if the filter is active
          
        
                $this->crud->addClause('where', 'category', 'LIKE', "%$value%");
            });
        $this->crud->addFilter([
            'type' => 'select2',
            'name' => 'faculty',
            'label' => trans('base.faculty')
        ],
            $facults,
            function ($value) { // if the filter is active
          
        
                $this->crud->addClause('where', 'faculty_id', 'LIKE', "$value");
            });
    }

    public function store(StoreRequest $request)
    {
        // your additional operations before save here
        $redirect_location = parent::storeCrud($request);
        // your additional operations after save here
        // use $this->data['entry'] or $this->crud->entry
        return $redirect_location;
    }

    public function update(UpdateRequest $request)
    {
        // your additional operations before save here
        $redirect_location = parent::updateCrud($request);
        // your additional operations after save here
        // use $this->data['entry'] or $this->crud->entry
        return $redirect_location;
    }
}
