<?php

namespace App\Http\Controllers\Auth;

use App\Models\Log;
use Backpack\Base\app\Http\Controllers\Controller;
use Carbon\Carbon;
use Illuminate\Foundation\Auth\AuthenticatesUsers;
use Illuminate\Http\Request;
use Auth;
use App\User;

class LoginController extends Controller
{
    protected $data = []; // the information we send to the view

    /*
    |--------------------------------------------------------------------------
    | Login Controller
    |--------------------------------------------------------------------------
    |
    | This controller handles authenticating users for the application and
    | redirecting them to your home screen. The controller uses a trait
    | to conveniently provide its functionality to your applications.
    |
    */
    use AuthenticatesUsers {
        logout as defaultLogout;
    }

    /**
     * Create a new controller instance.
     *
     * @return void
     */
    public function __construct()
    {

        $guard = backpack_guard_name();

        $this->middleware("guest:$guard", ['except' => 'logout']);

        // ----------------------------------
        // Use the admin prefix in all routes
        // ----------------------------------

        // If not logged in redirect here.
        $this->loginPath = property_exists($this, 'loginPath') ? $this->loginPath
            : backpack_url('login');
        // \Session::flush();


        // Redirect here after successful login.
        $this->redirectTo = property_exists($this, 'redirectTo') ? $this->redirectTo
            : backpack_url('dashboard');

        // Redirect here after logout.
        $this->redirectAfterLogout = property_exists($this, 'redirectAfterLogout') ? $this->redirectAfterLogout
            : backpack_url();
    }

    /**
     * Return custom username for authentication.
     *
     * @return string
     */
    public function username()
    {
        return backpack_authentication_column();
    }

    public function login(Request $request)
    {
        $this->data = [];
      //validate the fields....
      \Session::flush();
      $credentials = [ 'email' => $request->email , 'password' => $request->password ];
        
      if(Auth::attempt($credentials,$request->remember)){ // login attempt
        //login successful, redirect the user to your preferred url/route...
          if(Auth::user()->hasAnyRole(['Lecturer', 'Coordinator'])){
              Log::insert([
                  'user_full_name' => Auth::user()->full_name,
                  'user_role' => Auth::user()->role,
                  'type' => Log::AVAILABLE_TYPE['auth'],
                  'statement_id' => 0,
                  'created_at' => Carbon::now()
              ]);
          }
        return redirect('/dashboard');
      }else{
          $errors = [];
          if(User::where('email', '=', $request->email)->count() < 1){
              $errors['email'] = $request->email;
          } else {
              $errors['password'] = $request->password;
          }
        return redirect('/login')->withErrors($errors);
      }

      //login failed...
    }

    /**
     * Log the user out and redirect him to specific location.
     *
     * @param \Illuminate\Http\Request $request
     *
     * @return \Illuminate\Http\Response
     */
    public function logout(Request $request)
    {
        // Do the default logout procedure
        $this->guard()->logout();

        // And redirect to custom location
        return redirect($this->redirectAfterLogout);
    }

    /**
     * Get the guard to be used during logout.
     *
     * @return \Illuminate\Contracts\Auth\StatefulGuard
     */
    protected function guard()
    {
        return backpack_auth();
    }

    // -------------------------------------------------------
    // Laravel overwrites for loading backpack views
    // -------------------------------------------------------

    /**
     * Show the application login form.
     *
     * @return \Illuminate\Http\Response
     */
    public function showLoginForm()
    {

        $this->data['title'] = trans('backpack::base.login'); // set the page title
        $this->data['username'] = $this->username();
        //dd($this->data);
        return view('backpack::auth.login', $this->data);
    }
}
