<?php


namespace App\Http\Controllers;


use App\Extraction;

use App\Models\Student;
use Backpack\Settings\app\Models\Setting;
use Illuminate\Http\Request;

use Illuminate\Support\Arr;

use Illuminate\Support\Facades\App;
use Illuminate\Support\Facades\Auth;


class ExtractionController extends Controller {


    public function index(Student $student)
    {
        App::setLocale(session('language', 'ka'));
        $extractions = Extraction::where('student_id', $student->id)->get();

        foreach($extractions as $extraction){
            $orderToText = $this->converterOrderType($extraction->order_type, $this->getLocaleLang(), 'slug', 'name');;
            if($orderToText == null) $orderToText = $extraction->order_type;
            $extraction->order_type = $orderToText;
        }
        return view('admin.extractions', compact(
            'student',
            'extractions'
        ));
    }

    public function create(Student $student, Request $request){
        if (!$this->access_permission())
            return null;
        if ($request->isMethod('post')) {
            $param = $request->request->all();
            unset($param['_token']);
            $param['student_id'] = $student->id;
            $param['order_type'] = $this->converterOrderType($param['order_type'], $this->getLocaleLang(), 'name', 'slug');
            Extraction::insert($param);
            return redirect()->route('extractions', ['student' => $student]);
        }
        $typeExtraction = $this->getTypesExtraction();

        return view('admin.extractions_create', compact('student', 'typeExtraction'));
    }

    public function delete(Student $student, Extraction $extraction){
        if (!$this->access_permission())
            return null;
        $extraction->delete();
        return redirect()->route('extractions', ['student' => $student]);
    }

    public function edit(Student $student, Extraction $extraction, Request $request){
        if (!$this->access_permission())
            return null;
        if ($request->isMethod('post')) {
            $param = $request->request->all();
            unset($param['_token']);
            $param['order_type'] = $this->converterOrderType($param['order_type'], $this->getLocaleLang(), 'name', 'slug');;
            $extraction->update($param);
            return redirect()->route('extractions', ['student' => $student]);
        }
        $locale = $this->getLocaleLang();
        $extractionType = $this->converterOrderType($extraction->order_type, $locale, 'slug', 'name');
        $typeExtraction = $this->getTypesExtraction();
        return view('admin.extractions_edit', compact('extraction', 'student', 'extractionType', 'typeExtraction'));
    }

    public function getTypesExtraction(){
        $locale = $this->getLocaleLang();
        $typeExtraction = json_decode(Setting::where('key', 'orderType')->first()->value);
        foreach ($typeExtraction as $key => $value){
            if($key != $locale){
                unset($typeExtraction->$key);
                continue;
            }
            $typeExtraction->$key = json_decode($value);
        }
        return $typeExtraction->$locale;
    }

    /**
     * @param string $text
     * @param string $langLocale
     * @param $equal // с чем сравнивать slug, name
     * @param $return // какое значение возвращать slug, name
     * @return null
     */
    public function converterOrderType(string $text, string $langLocale, $equal, $return){
        $typeExtraction = json_decode(Setting::where('key', 'orderType')->first()->value);
        foreach ($typeExtraction as $lang => $array){
            if($langLocale != $lang and $langLocale != 'all'){
                unset($typeExtraction->$lang);
                continue;
            }
            $typeExtraction->$lang = json_decode($array);
            foreach($typeExtraction->$lang as $value) {
                if ($value->{$equal} == $text) {
                    return $value->{$return};
                }
            }
        }
        return null;
    }

    public function getLocaleLang(){
        return App::getLocale() == "en" ? "ru" : App::getLocale();
    }


    public function access_permission()

    {

        return Auth::user()->hasAnyRole(['Developer', 'SuperAdmin', 'Coordinator']);

    }

}
