<?php

namespace App\Http\Controllers;

use Alert;
use App\Models\Student;
use App\Models\UserAttribute;
use Backpack\Base\app\Http\Controllers\Controller;
use Backpack\Base\app\Http\Requests\ChangePasswordRequest;
use Illuminate\Support\Facades\Hash;
use App\Http\Requests\AccountInfoRequest;
use Illuminate\Support\Facades\Storage;

class MyAccountController extends Controller
{
    protected $data = [];

    public function __construct()
    {
        $this->middleware(backpack_middleware());
    }

    /**
     * Show the user a form to change his personal information.
     */
    public function getAccountInfoForm()
    {
        $student = Student::where('id',$this->guard()->user()->id)->first();

        $this->data['title'] = trans('backpack::base.my_account');
        $this->data['user'] = $this->guard()->user();
        if( !empty( $phone_number = UserAttribute::where('user_id', $this->guard()->user()->id)
            ->where('key', 'phone_number')
            ->first()) ){
            $this->data['phone_number'] = $phone_number->value;
        }

        if( !empty($student) && !empty( $faculty = $student->faculties->last()) ){
            $this->data['faculty'] = $faculty->name;
        }

        if( !empty($student) && !empty( $specialisation = $student->specialisations->last()) ){
            $this->data['specialisation'] = $specialisation->name;
        }
        if( !empty($student) && !empty( $group = $student->groups->last()) ){
            $this->data['group'] = $group->name;
        }

        if( !empty( $avatar = UserAttribute::where('user_id', $this->guard()->user()->id)
            ->where('key', 'avatar')
            ->first()) ){
            $this->data['avatar'] = $avatar->value;
        }


        return view('backpack::auth.account.update_info', $this->data);
    }

    /**
     * Save the modified personal information for a user.
     */
    public function postAccountInfoForm(AccountInfoRequest $request)
    {
        $result = $this->guard()->user()->update([
            'email' => $request->get('email'),
        ]);

        $phone_number = UserAttribute::where('user_id', $this->guard()->user()->id)
            ->where('key', 'phone_number')->update(['value' => $request->get('phone_number')]);
        if(!$phone_number){
            $phone_number = new UserAttribute([
                'user_id' => $this->guard()->user()->id,
                'type' => 'student',
                'key' => 'phone_number',
                'value' => $request->get('phone_number')
            ]);
            $phone_number->save();
        }

        if( !empty($file = $request->file('avatar')) && $file->isValid() ){
            $imageName = time().'.'.$request->file('avatar')->getClientOriginalExtension();
            $request->file('avatar')->storeAs('avatars', $imageName, 'public');

            $avatar = UserAttribute::where('user_id', $this->guard()->user()->id)
                ->where('key', 'avatar')->first();

            Storage::disk('public')->delete('avatars/'.$avatar->value);

            $avatar = UserAttribute::where('user_id', $this->guard()->user()->id)
                ->where('key', 'avatar')
                ->update(['value' => $imageName]);

            if(!$avatar){
                $avatar = new UserAttribute([
                    'user_id' => $this->guard()->user()->id,
                    'type' => 'student',
                    'key' => 'avatar',
                    'value' => $imageName
                ]);
                $avatar->save();
            }
        }

        if ($result) {
            Alert::success(trans('backpack::base.account_updated'))->flash();
        } else {
            Alert::error(trans('backpack::base.error_saving'))->flash();
        }

        return redirect()->back();
    }

    /**
     * Show the user a form to change his login password.
     */
    public function getChangePasswordForm()
    {
        $this->data['title'] = trans('backpack::base.my_account');
        $this->data['user'] = $this->guard()->user();

        return view('backpack::auth.account.change_password', $this->data);
    }

    /**
     * Save the new password for a user.
     */
    public function postChangePasswordForm(ChangePasswordRequest $request)
    {
        $user = $this->guard()->user();
        $user->password = Hash::make($request->new_password);

        if ($user->save()) {
            Alert::success(trans('backpack::base.account_updated'))->flash();
        } else {
            Alert::error(trans('backpack::base.error_saving'))->flash();
        }

        return redirect()->back();
    }

    /**
     * Get the guard to be used for account manipulation.
     *
     * @return \Illuminate\Contracts\Auth\StatefulGuard
     */
    protected function guard()
    {
        return backpack_auth();
    }
}
