<?php

namespace App\Http\Controllers;

use App\Models\Statement;
use Illuminate\Database\Eloquent\Builder;
use Illuminate\Http\Request;
use App\Models\Subject;
use Illuminate\Support\Facades\Auth;

class SubjectController extends Controller
{
    public function studentSubjectList($id)
    {
        dd($id);
    }

    public function getFacultySubjects(Request $request)
    {
        $subjects = Subject::whereHas('faculties', function (Builder $query) use ($request){
            $query->where('id', $request->get('id'));
        })->get()->map->only('id', 'name');

        return response()->json(['subjects' => $subjects],200);
    }

    public function setFacultySubject(Request $request)
    {
        if ( !Auth::user()->hasAnyRole(['Developer','SuperAdmin', 'Coordinator'])){
            return response()->json(['message' => __('base.not_authorized'), 401]);
        }

        $student_id = $request->get('student_id');
        $subject_id = $request->get('subject_id');

        if (!empty($subject_id)){
            $statement = new Statement();

            $statement->subject_id = $subject_id;
            $statement->specialisation_id = 10;
            $statement->setTranslation('number', 'ru', 'Individual');

            $statement->save();

            if(!empty($student_id)){
                $student_for_sync = [$student_id => ['type' => 'student']];

                $statement->students()->sync($student_for_sync);
                $statement->individual_students()->sync([$student_id]);
            }
            $subject = $statement->subject;
            $faculty = $subject->faculties->first();
            $result = [
                'statement_id' => $statement->id,
                'faculty_name' => $faculty->name,
                'subject_name' => $subject->name,
            ];
            return response()->json([
                'id' => $statement->id,
                'result' => $result
            ],201);
        }

        return response()->json(['message' => __('base.not_changed')], 304);
    }

    public function deleteFacultySubject(Request $request)
    {
        if ( !Auth::user()->hasAnyRole(['Developer','SuperAdmin', 'Coordinator'])){
            return response()->json(['message' => __('base.not_authorized'), 401]);
        }

        Statement::where('id', $request->get('statement_id'))->delete();
        return response()->json(['message' => 'Successful deleted!', 'subject_id' => $request->get('statement_id')], 200);
    }

    public function getAllStudentSubjectsList(Request $request)
    {
        if ( !Auth::user()->hasAnyRole(['Developer','SuperAdmin', 'Coordinator'])){
            return response()->json(['message' => __('base.not_authorized'), 401]);
        }
        $student_id = $request->get('student_id');
        if(!empty($student_id)){

        }

        $statements = Statement::whereHas('individual_students', function(Builder $query) use ($student_id){
            $query->where('id', $student_id);
        })->with(['subject', 'subject.faculties'])->get(); //->map->only('id', 'name');

        $result = [];
        foreach ($statements as $statement){
            $result[$statement->id] = [
                'statement_id' => $statement->id,
                'faculty_name' => $statement->subject->faculties->first()->name,
                'subject_name' => $statement->subject->name
            ];
        }

        return response()->json(['subjects' => $result], 200);
    }
}
