<?php

namespace App\Http\Controllers;

use App\Models\{ProgramDocument,Timetable, BackpackUser, UserAttribute, Group};
use App\Models\Syllabus;
use Illuminate\Http\Request;
use Illuminate\Support\Arr;

class SyllabusController extends Controller
{
    public function show()
    {
        if(\Auth::user()->hasRole('')){
            $u = BackpackUser::find(\Auth::id());
            $fc = $u::realFacultyIDs();

            if(!empty($fc)){
                $return = Syllabus::orderBy('category')->whereIn('faculty_id', $fc)->get()->groupBy(['faculty_id', function ($item){
                    return $item['category'];
                }]);
            } else {
                $return = [];
            }
        } else {
            $return = Syllabus::orderBy('category')->get()->groupBy(['faculty_id', function ($item){
                return $item['category'];
            }]);
        }

        $slugs = Arr::pluck(json_decode(config('settings.syllabus_categories'), true), 'name', 'slug');
        return view('admin.syllabuses', [
            'return' => $return,
            'slug_to_cat' => $slugs,
            'url_prefix' => 'storage/syllabuses/'
        ]);
    }

    public function showPrograms()
    {
        if(\Auth::user()->hasRole('')){
            $u = BackpackUser::find(\Auth::id());
            $fc = $u::realFacultyIDs();

            if(!empty($fc)){
                $return = ProgramDocument::orderBy('category')->whereIn('faculty_id', $fc)->get()->groupBy(['faculty_id', function ($item){
                    return $item['category'];
                }]);
            } else {
                $return = [];
            }
        } else {
            $return = ProgramDocument::orderBy('category')->get()->groupBy(['faculty_id', function ($item){
                return $item['category'];
            }]);
        }

        $slugs = Arr::pluck(json_decode(config('settings.studyPrograms'), true), 'name', 'slug');
        return view('admin.syllabuses', [
            'return' => $return,
            'slug_to_cat' => $slugs,
            'url_prefix' => 'storage/program_documents/'
        ]);
    }

    public static function showTimetables()
    {
        if(\Auth::user()->hasRole('Student')){
            $u = BackpackUser::find(\Auth::id());
            $fc = $u->realFacultyIDs();

            if(!empty($fc)){
                $return = Timetable::orderBy('category')->whereIn('faculty_id', $fc)->get()->groupBy(['faculty_id', function ($item){
                    return $item['category'];
                }]);
            } else {
                $return = [];
            }

        } else {
            $return = Timetable::orderBy('category')->get()->groupBy(['faculty_id', function ($item){
                return $item['category'];
            }]);
        }

        $slugs = Arr::pluck(json_decode(config('settings.studyPrograms'), true), 'name', 'slug');
        return view('admin.timetables', [
            'return' => $return,
            'slug_to_cat' => $slugs,
            'url_prefix' => 'storage/timetable/'
        ]);
    }

    function showStudentsUpdate(){
        $statuses = Arr::pluck(json_decode(config('settings.statuses'), true), 'name', 'slug');
        $facs = \App\Models\Specialisation::whereHas('faculty')->get();
        $group = Group::get()
                ->pluck('name', 'id')
                ->toArray();
        $students = \App\Models\Student::whereHas('specialisations')->get();
        
        return view('admin.studentsUpdate', compact('statuses', 'facs', 'group', 'students'));   
    }

    function StudentsUpdateAll(Request $r){
        UserAttribute::where('key', 'status')
                ->where('value', '!=', 'deduct')
                ->where('value', '!=', 'graduate')
                ->update(['value' => $r->input('status')]);
    }
    
    function StudentsUpdateFaculty(Request $r){
        // Костиль наше все!
        UserAttribute::leftjoin('user_specialisation', 'user_attributes.user_id', '=', 'user_specialisation.user_id')
                ->where('user_specialisation.specialisation_id', $r->input('faculty'))
                ->where('user_attributes.key', 'status')
                ->where('user_attributes.value', '!=', 'deduct')
                ->where('user_attributes.value', '!=', 'graduate')
                ->where('user_attributes.value', '!=', 'graduate')
                ->update(['value' => $r->input('status')]);
    }

    function StudentsUpdateGroup(Request $r){
        UserAttribute::leftjoin('group_user', 'user_attributes.user_id', '=', 'group_user.user_id')
                ->where('group_user.group_id', $r->input('group'))
                ->where('user_attributes.key', 'status')
                ->where('user_attributes.value', '!=', 'deduct')
                ->where('user_attributes.value', '!=', 'graduate')
                ->where('user_attributes.value', '!=', 'graduate')
                ->update(['value' => $r->input('status')]);
                // ->update(['value' => $r->input('status')]);
    }

    function StudentsUpdateStudents(Request $r){
        // dd($r->all());
        $e =UserAttribute::leftjoin('users', 'user_attributes.user_id', '=', 'users.id')
                ->whereIn('users.id', $r->input('students'))
                ->where('user_attributes.key', 'status')
                ->where('user_attributes.value', '!=', 'deduct')
                ->where('user_attributes.value', '!=', 'graduate')
                ->where('user_attributes.value', '!=', 'graduate')
                ->update(['value' => $r->input('status')]);
                // ->get();
        // dd($e);
                // ->update(['value' => $r->input('status')]);
    }
    
}
