<?php

namespace App\Traits;

use Backpack\CRUD\ModelTraits\SpatieTranslatable\HasTranslations as OriginalHasTranslation;

trait HasTranslations
{
    use OriginalHasTranslation;

    public function getTranslation(string $key, string $locale, bool $useFallbackLocale = true)
    {

        $locale = \App::getLocale();

        $translations = $this->getTranslations($key);

        if (empty($translations[$locale])){
            $translation = '';
            if($locale == 'ru' && !empty($translations['ka'])){
                $translation = $translations['ka'];
            }
            if($locale == 'ka' && !empty($translations['ru'])){
                $translation = $translations['ru'];
            }
            if($locale == 'en' && !empty($translations['en'])){
                $translation = $translations['en'];
            }
            if($translation == null and !empty($translations['en'])){
                $translation = $translations['en'];
            }
        } else {
            $translation = $translations[$locale];
        }
        if(!$translation)   if(!empty($translations['ru'])) $translation = $translations['ru']; elseif(!empty($translations['ka'])) $translation = $translations['ka'];
        if ($this->hasGetMutator($key)) {
            return $this->mutateAttribute($key, $translation);
        }

        return $translation;
    }
    /**
     * Convert the model instance to an array.
     *
     * @return array
     */
    public function toArray()
    {
       $attributes = parent::toArray();
        foreach ($this->getTranslatableAttributes() as $field) {
            $attributes[$field] = $this->getTranslation($field, \App::getLocale());
        }
        return $attributes;
    }

    /**
     * Magic method to get the db entries already translated in the wanted locale.
     *
     * @param string $method
     * @param array $parameters
     * @return
     */
    public function __call($method, $parameters)
    {
        switch ($method) {
            // translate all find methods
            case 'find':
            case 'findOrFail':
            case 'findMany':
            case 'findBySlug':
            case 'findBySlugOrFail':

                $translation_locale = \Request::input('locale', \App::getLocale());
//dd($parameters);
                if ($translation_locale) {
                    $item = parent::__call($method, $parameters);
                    if ($item instanceof \Traversable) {
                        foreach ($item as $instance) {
                            $instance->setLocale($translation_locale);
                        }
                    } elseif ($item) {
                        $item->setLocale($translation_locale);
                    }
                    return $item;
                }

                return parent::__call($method, $parameters);
                break;

            // do not translate any other methods
            default:
                return parent::__call($method, $parameters);
                break;
        }
    }
}
