<?php

namespace App\Models;

use App\Traits\HasTranslations;
use Illuminate\Database\Eloquent\Model;
use Backpack\CRUD\CrudTrait;

class Faculty extends Model
{
    use CrudTrait;
    use HasTranslations;

    /*
    |--------------------------------------------------------------------------
    | GLOBAL VARIABLES
    |--------------------------------------------------------------------------
    */

    protected $table = 'faculties';
    protected $primaryKey = 'id';
    public $timestamps = true;
    protected $guarded = ['id'];
    protected $fillable = [
        'name',
        'description'
    ];
    public $translatable = [
        'name',
        'description'
    ];

    /*
    |--------------------------------------------------------------------------
    | FUNCTIONS
    |--------------------------------------------------------------------------
    */
    protected function asJson($value)
    {
        return json_encode($value, JSON_UNESCAPED_UNICODE);
    }

    public static function listForIndividualProgram(){
        return json_encode(Faculty::all()->map->only('id', 'name'), JSON_UNESCAPED_UNICODE);
    }

    /*
    |--------------------------------------------------------------------------
    | RELATIONS
    |--------------------------------------------------------------------------
    */

    public function specialisations(){
        return $this->hasMany('App\Models\Specialisation');
    }
    public function faculty_subject(){
        return $this->belongsToMany('App\Models\Subject', 'faculty_subject', 'faculty_id', 'subject_id');
    }
    public function faculty_user(){
        return $this->belongsToMany('App\User', 'user_faculty', 'faculty_id', 'user_id');
    }
    public function faculty_lecturer(){
        return $this->belongsToMany('App\Models\Lecturer', 'user_faculty', 'faculty_id', 'user_id');
    }
    public function syllabuses(){
        return $this->hasMany('App\Models\Syllabus');
    }
    /*
    |--------------------------------------------------------------------------
    | SCOPES
    |--------------------------------------------------------------------------
    */

    /*
    |--------------------------------------------------------------------------
    | ACCESORS
    |--------------------------------------------------------------------------
    */

    /*
    |--------------------------------------------------------------------------
    | MUTATORS
    |--------------------------------------------------------------------------
    */
}
