<?php



namespace App\Models;



use App\Traits\HasTranslations;

use Illuminate\Database\Eloquent\Model;

use Backpack\CRUD\CrudTrait;

use Illuminate\Http\Request;



class Group extends Model

{

    use CrudTrait;

    use HasTranslations;



    /*

    |--------------------------------------------------------------------------

    | GLOBAL VARIABLES

    |--------------------------------------------------------------------------

    */



    protected $table = 'groups';

    protected $primaryKey = 'id';

    public $timestamps = true;

    protected $guarded = ['id'];

    protected $fillable = [

        'specialisation_id',

        'program_slug',

        'name',

        'description',

        'stage',

        'semester',

    ];

    public $translatable = [

        'name',

        'description'

    ];



    /*

    |--------------------------------------------------------------------------

    | FUNCTIONS

    |--------------------------------------------------------------------------

    */

    protected function asJson($value)

    {

        return json_encode($value, JSON_UNESCAPED_UNICODE);

    }

    /*

    |--------------------------------------------------------------------------

    | RELATIONS

    |--------------------------------------------------------------------------

    */



    public function program()

    {

        return $this->belongsTo('App\Models\Program');

    }



    public function specialisation()

    {

        return $this->belongsTo('App\Models\Specialisation');

    }

    public function students()
    {
        return $this->belongsToMany('App\Models\Student', 'group_user', 'group_id', 'user_id')->withTimestamps()->withPivot('updated_at');
    }


    /*

    |--------------------------------------------------------------------------

    | SCOPES

    |--------------------------------------------------------------------------

    */



    /*

    |--------------------------------------------------------------------------

    | ACCESORS

    |--------------------------------------------------------------------------

    */

//    public static function groupsList() {

//        $options = Group::all()->pluck('name', 'id')->toArray();

//        return $options;

//    }

    /*

    |--------------------------------------------------------------------------

    | MUTATORS

    |--------------------------------------------------------------------------

    */

}

