<?php

namespace App\Models;

use Illuminate\Database\Eloquent\Model;
use Backpack\CRUD\CrudTrait;
use Illuminate\Database\Eloquent\Builder;
use Spatie\Permission\PermissionRegistrar;
use Illuminate\Support\Arr;

class Lecturer extends BackpackUser
{
    /*
    |--------------------------------------------------------------------------
    | GLOBAL VARIABLES
    |--------------------------------------------------------------------------
    */
    protected $table = 'users';
    protected $guard_name = 'backpack';

    protected $fillable = [
        'first_name',
        'last_name',
        'birth_date',
        'gender',
        'email',
        'password'
    ];
    public $translatable = [
        'first_name',
        'last_name'
    ];

    /*
    |--------------------------------------------------------------------------
    | FUNCTIONS
    |--------------------------------------------------------------------------
    */

    protected static function boot()
    {
        parent::boot();

        static::addGlobalScope('roles', function (Builder $builder) {
            $builder->role("Lecturer");
        });
    }

    public function viewLecturerBtn($crud = false)
    {
        return '<a class="btn btn-xs btn-default" target="_blank" href="http://google.com?q='.urlencode($this->text).'" data-toggle="tooltip" title="Just a demo custom button."><i class="fa fa-address-card"></i></a>';
    }
    public function whisperLecturerBtn($crud = false)
    {
        return '<a class="btn btn-xs btn-default" target="_blank" href="http://google.com?q='.urlencode($this->text).'" data-toggle="tooltip" title="Just a demo custom button."><i class="fa fa-comment"></i></a>';
    }
    /*
    |--------------------------------------------------------------------------
    | RELATIONS
    |--------------------------------------------------------------------------
    */
    public function statement_user()
    {
        return $this->belongsToMany('App\Models\UserAttribute', 'user_id')
            ->whereIn('key', $this->fields);
    }

    public function faculties(){
        return $this->belongsToMany('App\Models\Faculty', 'user_faculty', 'user_id', 'faculty_id');
    }
    public function user_attributes()
    {
        return $this->hasMany('App\Models\UserAttribute', 'user_id');
    }

    /*
    |--------------------------------------------------------------------------
    | SCOPES
    |--------------------------------------------------------------------------
    */

    /*
    |--------------------------------------------------------------------------
    | ACCESORS
    |--------------------------------------------------------------------------
    */
    public function getAcademicDegreeAttribute()
    {
        return $this->user_attributes()->where('key', 'academicDegree')->get('value');
    }
    public function getAcademicDegreeNameAttribute()
    {
        $academicDegree = Arr::pluck(json_decode(config('settings.academicDegree'), true), 'name', 'slug');
        if(!empty($this->user_attributes()->where('key', 'academicDegree')->first())){
            return $academicDegree[$this->user_attributes()->where('key', 'academicDegree')->first()->value];
        }
    }

    public function getFullNameAttribute()
    {
//        dd($this->getTranslations('first_name'));
        return "({$this->academic_degree_name}) {$this->last_name} {$this->first_name}";
    }

    /*
    |--------------------------------------------------------------------------
    | MUTATORS
    |--------------------------------------------------------------------------
    */
}
