<?php

namespace App\Models;

use Illuminate\Database\Eloquent\Model;
use Backpack\CRUD\CrudTrait;

class Mark extends Model
{
    use CrudTrait;

    /*
    |--------------------------------------------------------------------------
    | GLOBAL VARIABLES
    |--------------------------------------------------------------------------
    */

    protected $table = 'marks';
    protected $primaryKey = 'id';
    public $timestamps = true;
    protected $guarded = ['id'];
    protected $fillable = [
        'user_id',
        'subject_id',
        'statement_id',
        'mark_group_id',
        'value',
        'data'
    ];
    public $data;
    protected $with = [
        'mark_group'
    ];

    /*
    |--------------------------------------------------------------------------
    | FUNCTIONS
    |--------------------------------------------------------------------------
    */
    public static function toECTS($gpa)
    {
        if( $gpa >= 96){
            return '4,0';
        } elseif ($gpa >= 91 && $gpa < 96){
            return '3,7';
        } elseif ($gpa >= 87 && $gpa < 91){
            return '3,3';
        } elseif ($gpa >= 84 && $gpa < 87){
            return '3,0';
        } elseif ($gpa >= 81 && $gpa < 84){
            return '2,7';
        } elseif ($gpa >= 76 && $gpa < 81){
            return '2,4';
        } elseif ($gpa >= 71 && $gpa < 76){
            return '2,0';
        } elseif ($gpa >= 66 && $gpa < 71){
            return '1,7';
        } elseif ($gpa >= 61 && $gpa < 66){
            return '1,3';
        } elseif ($gpa >= 56 && $gpa < 61){
            return '1,0';
        } elseif ($gpa >= 51 && $gpa < 56){
            return '0,7';
        } elseif ($gpa < 51){
            return '0';
        }
    }

    public function getDataAttribute(){
        return json_decode($this->data, true);
    }

    public function toGpa($gpa)
    {
        if( $gpa >= 96){
            return 4.0;
        } elseif ($gpa >= 91 && $gpa < 96){
            return 3.7;
        } elseif ($gpa >= 87 && $gpa < 91){
            return 3.3;
        } elseif ($gpa >= 84 && $gpa < 87){
            return 3.0;
        } elseif ($gpa >= 81 && $gpa < 84){
            return 2.7;
        } elseif ($gpa >= 76 && $gpa < 81){
            return 2.4;
        } elseif ($gpa >= 71 && $gpa < 76){
            return 2.0;
        } elseif ($gpa >= 66 && $gpa < 71){
            return 1.7;
        } elseif ($gpa >= 61 && $gpa < 66){
            return 1.3;
        } elseif ($gpa >= 56 && $gpa < 61){
            return 1.0;
        } elseif ($gpa >= 51 && $gpa < 56){
            return 0.7;
        } elseif ($gpa < 51){
            return 0;
        }

        // if( 96 <= $gpa ){
        //     return 4.0;
        // } elseif (91 <= $gpa && $gpa <= 95){
        //     return 3.7;
        // } elseif (87 <= $gpa && $gpa <= 90){
        //     return 3.3;
        // } elseif (84 <= $gpa && $gpa <= 86){
        //     return 3.0;
        // } elseif (81 <= $gpa && $gpa <= 83){
        //     return 2.7;
        // } elseif (76 <= $gpa && $gpa <= 80){
        //     return 2.7;
        // } elseif (71 <= $gpa && $gpa <= 75){
        //     return 2.0;
        // } elseif (66 <= $gpa && $gpa <= 70){
        //     return 1.7;
        // }elseif (61 <= $gpa && $gpa <= 65){ // Such values doen't exist in excel
        //     return 1.3;
        // } elseif (56 <= $gpa && $gpa <= 60){
        //     return 1.0;
        // } elseif (51 <= $gpa && $gpa <= 55){
        //     return 0.7;
        // } else {
        //     return 0;
        // }
    }

    /*
    |--------------------------------------------------------------------------
    | RELATIONS
    |--------------------------------------------------------------------------
    */

    public function student()
    {
        return $this->belongsTo('App\Models\Student', 'user_id');
    }

    public function subject()
    {
        return $this->belongsTo('App\Models\Subject');
    }

    public function statement()
    {
        return $this->belongsTo('App\Models\Statement');
    }

    public function mark_group()
    {
        return $this->belongsTo('App\Models\MarkGroup');
    }

    /*
    |--------------------------------------------------------------------------
    | SCOPES
    |--------------------------------------------------------------------------
    */

    /*
    |--------------------------------------------------------------------------
    | ACCESORS
    |--------------------------------------------------------------------------
    */
    public function getEctsAttribute()
    {
        $credits = $this->statement->count;
        if($this->mark_group_id == 9){
            $gpa = self::toGpa($this->value);
            return $credits * $gpa;
        } else {
            return 0;
        }
    }

    public function getFuckingvariableAttribute(){
        $grade = $this->value;
        if($grade <= 100 and $grade >= 91)
            $grade = 'A';
        elseif($grade <= 90 and $grade >= 81)
            $grade = 'B';
        elseif($grade <= 80 and $grade >= 71)
            $grade = 'C';
        elseif($grade <= 70 and $grade >= 61)
            $grade = 'D';
        elseif($grade <= 60 and $grade >= 51)
            $grade = 'E';
        elseif($grade == 0 or $grade <= 50)
            $grade = 'F';
       // if($marsk['final-exam-iii'] < 20)
           // $grade = 'FX';
       //// if($marsk['additional-exam'] < 20)
         //   $grade = 'F';
        
        return $grade;
    }

    public function getGpaAttribute()
    {
        $credits = $this->statement->count;
        if($this->mark_group_id == 9){
            return self::toGpa($this->value);
        } else {
            return 0;
        }
    }

    public function getCreditsAttribute()
    {
        return $this->statement->count;
    }
    /*
    |--------------------------------------------------------------------------
    | MUTATORS
    |--------------------------------------------------------------------------
    */
}
