<?php

namespace App\Models;

use App\Traits\HasTranslations;
use Illuminate\Database\Eloquent\Model;
use Backpack\CRUD\CrudTrait;
use Illuminate\Support\Arr;
use Illuminate\Support\Facades\Storage;

class ProgramDocument extends Model
{
    use CrudTrait;
    use HasTranslations;

    /*
    |--------------------------------------------------------------------------
    | GLOBAL VARIABLES
    |--------------------------------------------------------------------------
    */

    protected $table = 'programs_documents';
    // protected $primaryKey = 'id';
    // public $timestamps = false;
    // protected $guarded = ['id'];
    protected $fillable = [
        'name',
        'category',
        'faculty_id',
        'file'
    ];
    // protected $hidden = [];
    // protected $dates = [];

    public $translatable = [
        'name',
    ];

    public static function boot()
    {
        parent::boot();
        static::deleting(function($obj) {
            Storage::disk('program_document')->delete($obj->file);
        });
    }
    /*
    |--------------------------------------------------------------------------
    | FUNCTIONS
    |--------------------------------------------------------------------------
    */
    public function getCategory()
    {
        $category = $this->category;
        return Arr::pluck(json_decode(config('settings.studyPrograms'), true), 'name', 'slug')[$category] ?? $category;
    }
    public function getFacultyName()
    {
        return $this->faculty->name;
    }
    /*
    |--------------------------------------------------------------------------
    | RELATIONS
    |--------------------------------------------------------------------------
    */
    public function faculty()
    {
        return $this->belongsTo('App\Models\Faculty', 'faculty_id');
    }
    /*
    |--------------------------------------------------------------------------
    | SCOPES
    |--------------------------------------------------------------------------
    */

    /*
    |--------------------------------------------------------------------------
    | ACCESORS
    |--------------------------------------------------------------------------
    */

    /*
    |--------------------------------------------------------------------------
    | MUTATORS
    |--------------------------------------------------------------------------
    */
    public function setFileAttribute($value)
    {
        $attribute_name = "file";
        $disk = "program_document";
        $destination_path = "";

        $this->uploadFileToDisk($value, $attribute_name, $disk, $destination_path);
    }
}
