<?php

namespace App\Models;

use App\Traits\HasTranslations;
use Illuminate\Database\Eloquent\Model;
use Backpack\CRUD\CrudTrait;
use Illuminate\Support\Arr;

class Statement extends Model
{
    use CrudTrait;
    use HasTranslations;

    protected $attributes = [
        'form' => 'statement'
    ];

    /*
    |--------------------------------------------------------------------------
    | GLOBAL VARIABLES
    |--------------------------------------------------------------------------
    */

    protected $table = 'statements';
    protected $primaryKey = 'id';
    public $timestamps = true;
    protected $guarded = ['id'];
    protected $fillable = [
        'specialisation_id',
        'group_id',
        'subject_id',
        'count',
        'hours',
        'required',
        'type',
        'step',
        'semester',
        'number',
        'closed_at',
        'form',
        'students',
        'year_study',
        'created_by'
    ];
    protected $hidden = [];
    // protected $dates = [
    //     'closed_at'
    // ];
    public $translatable = [
        'number'
    ];

    public const ALL_FORMS = [
        'bachelor',
        'practice',
        'statement',
        'osce'
    ];
    
    public static function created($callback){
        $this->created_by = auth()->user()->id ?? 2;
        $this->save();
        return parent::creating($callback);
    }

    /*
    |--------------------------------------------------------------------------
    | FUNCTIONS
    |--------------------------------------------------------------------------
    */

    public function print($crud = false)
    {

        $test = $this->form . 's/';
        ob_start(); ?>
        <div class="btn-group">
            <button type="button" class="btn btn-default btn-xs dropdown-toggle" data-toggle="dropdown" aria-haspopup="true" aria-expanded="false">
                <i class="fa fa-print"></i> <?= __('base.statement.print') ?> <span class="caret"></span>
            </button>
            <ul class="dropdown-menu">
                <li><a href="<?= backpack_url($test . $this->id . '/print/ru') ?>"><?= __('base.ru') ?></a></li>
                <li><a href="<?= backpack_url($test . $this->id . '/print/ka') ?>"><?= __('base.ka') ?></a></li>
                <li><a href="<?= backpack_url($test . $this->id . '/print/en') ?>"><?= __('base.en') ?></a></li>
            </ul>
        </div>
        <?php
        return ob_get_clean();
    }

    public function cloneStatement($crud = false)
    {
        $test = $this->form . 's/';
        ob_start(); ?>
            <a href="<?= backpack_url($test . $this->id . '/clone') ?>" class="btn btn-xs btn-default" ><i class="fa fa-clone"></i> <?= __('base.statement.clone') ?></a>
        <?php
        return ob_get_clean();
    }

    public function edit($crud = false)
    {
        $test = $this->form . 's/';
        ob_start(); ?>
            <a href="<?= backpack_url($test . $this->id . '/rating') ?>" class="btn btn-xs btn-default" ><i class="fa fa-check"></i> <?= __('base.statement.marks') ?></a>
        <?php
        return ob_get_clean();
    }

    public function getStatementRequirement()
    {
        $statement_required = $this->required;
        return Arr::pluck(json_decode(config('settings.required'), true), 'name', 'slug')[$statement_required] ?? $statement_required;
    }


    /*
    |--------------------------------------------------------------------------
    | RELATIONS
    |--------------------------------------------------------------------------
    */

    public function students()
    {
        return $this->belongsToMany('App\Models\Student', 'statement_user', 'statement_id', 'user_id')
            ->wherePivot('type', 'student');
    }

    public function individual_students()
    {
        return $this->belongsToMany('App\Models\Student', 'individual_statement_student', 'statement_id', 'student_id');
    }

    public function form()
    {
        return true;
    }

    public function lecturers()
    {
        return $this->belongsToMany('App\Models\Lecturer', 'statement_user', 'statement_id', 'user_id')
            ->wherePivot('type', 'lecturer');
    }

    public function specialisation()
    {
        return $this->belongsTo('App\Models\Specialisation');
    }

    public function group()
    {
        return $this->belongsTo('App\Models\Group');
    }

    public function subject()
    {
        return $this->belongsTo('App\Models\Subject');
    }

    public function supervision()
    {
        return $this->hasOne(Supervision::class);
    }

    /*
    |--------------------------------------------------------------------------
    | SCOPES
    |--------------------------------------------------------------------------
    */

    /*
    |--------------------------------------------------------------------------
    | ACCESORS
    |--------------------------------------------------------------------------
    */

    public function getRequiredNameAttribute()
    {
        return __('base.required.' . $this->required) ?? $this->required;
    }

    public function getTypeNameAttribute()
    {
        return Arr::pluck(json_decode(config('settings.type'), true), 'name', 'slug')[$this->type] ?? $this->type;
    }

    public function getStepNameAttribute()
    {
        return Arr::pluck(json_decode(config('settings.stage'), true), 'name', 'slug')[$this->step] ?? $this->step;
    }

    public function getSemesterNameAttribute()
    {
        return Arr::pluck(json_decode(config('settings.semester'), true), 'name', 'slug')[$this->semester] ?? $this->semester;
    }

    public function getIndividualStudentAttribute()
    {
        return $this->individual_students()->first();
    }

    /*
    |--------------------------------------------------------------------------
    | MUTATORS
    |--------------------------------------------------------------------------
    */
}
