<?php

namespace App\Models;

use Backpack\CRUD\ModelTraits\SpatieTranslatable\HasTranslations;
use Illuminate\Database\Eloquent\Builder;
use Illuminate\Database\Eloquent\Model;
use Backpack\CRUD\CrudTrait;
use Illuminate\Support\Arr;


class Student extends BackpackUser
{
    /*
    |--------------------------------------------------------------------------
    | GLOBAL VARIABLES
    |--------------------------------------------------------------------------
    */

    protected $table = 'users';
    protected $guard_name = 'backpack';
    protected $fillable = [
        'first_name',
        'last_name',
        'birth_date',
        'gender',
        'email',
        'password'
    ];
    public $translatable = [
        'first_name',
        'last_name'
    ];

    /*
    |--------------------------------------------------------------------------
    | FUNCTIONS
    |--------------------------------------------------------------------------
    */
    protected static function boot()
    {
        parent::boot();

        static::addGlobalScope('role', function (Builder $builder) {
            $builder->role('Student');
        });
    }
    
    public static function created($callback){
        self::fillTranslatable();
        return parent::created($callback);
    }
    
    public static function updated($callback){
        self::fillTranslatable();
        return parent::updated($callback);
    }
    
    
    public static function fillTranslatable(){
        $translationsFirstName = $this->getTranslations('first_name');
        $translationsLastName = $this->getTranslations('last_name');
        $allLang = ['ru', 'en', 'ka'];
        $last_name = $this->last_name;
        $first_name = $this->first_name;
        foreach($allLang as $lang){
            if($translationsLastName[$lang] == null){
                $this->setTransaltions('last_name', $lang, $last_name);
            }
            if($translationsFirstName[$lang] == null){
                $this->setTransaltions('first_name', $lang, $first_name);
            }
        }
        $this->save();
    }
    public function getCardId()
    {
        return $this->user_attributes()->where('key', 'studentCard')->value('value');
    }
    public function getStudentStatusView()
    {
        $user_status = $this->user_attributes()->where('key', 'status')->value('value');
        $status_text = Arr::pluck(json_decode(config('settings.statuses'), true), 'name', 'slug')[$user_status] ?? $user_status;
        if($user_status == 'active'){
            return '<span class="label label-success">'.$status_text.'</span>';
        }
        if($user_status == 'suspend'){
            return '<span class="label label-warning">'.$status_text.'</span>';
        }
        if($user_status == 'deduct'){
            return '<span class="label label-danger">'.$status_text.'</span>';
        }
        if(!empty($status_text)){
            return '<span class="label label-default">'.$status_text.'</span>';
        }
    }
    public function getStudentGender()
    {
        $user_gender = $this->gender;
        return Arr::pluck(json_decode(config('settings.gender'), true), 'name', 'slug')[$user_gender] ?? $user_gender;
    }

    public function personalProgramButton($crud = false)
    {
        ob_start(); ?>
        <a href="<?= backpack_url('statements/' . $this->id . '/rating') ?>" class="btn btn-xs btn-default" ><i class="fa fa-check"></i> Оценки</a>
        <?php
        return ob_get_clean();
    }

    public function print($crud = false)
    {
        ob_start(); ?>
        <div class="btn-group">
            <button type="button" class="btn btn-default btn-xs dropdown-toggle" data-toggle="dropdown" aria-haspopup="true" aria-expanded="false">
                <i class="fa fa-print"></i> <?= __('base.extractions.extraction_print') ?> <span class="caret"></span>
            </button>
            <ul class="dropdown-menu">
                <li><a href="<?= backpack_url('statements/' . $this->id . '/printExtraction/ru') ?>"><?= __('base.ru')?></a></li>
                <li><a href="<?= backpack_url('statements/' . $this->id . '/printExtraction/ka') ?>"><?= __('base.ka')?></a></li>
                <li><a href="<?= backpack_url('statements/' . $this->id . '/printExtraction/en') ?>"><?= __('base.en')?></a></li>
            </ul>
        </div>
        <?php
        return ob_get_clean();
    }

    public function edit($crud = false)
    {
        ob_start(); ?>
        <a href="<?= backpack_url('statements/' . $this->id . '/rating') ?>" class="btn btn-xs btn-default" ><i class="fa fa-check"></i> Оценки</a>
        <?php
        return ob_get_clean();
    }
    /*
    |--------------------------------------------------------------------------
    | RELATIONS
    |--------------------------------------------------------------------------
    */
    public function faculties(){
        return $this->belongsToMany('App\Models\Faculty', 'user_faculty', 'user_id');
    }

    public function specialisations(){
        return $this->belongsToMany('App\Models\Specialisation', 'user_specialisation', 'user_id');
    }

    public function groups(){
        return $this->belongsToMany('App\Models\Group', 'group_user', 'user_id')->withTimestamps()->withPivot('updated_at');
    }

    public function user_attributes()
    {
        return $this->hasMany('App\Models\UserAttribute', 'user_id');
    }

    public function marks()
    {
        return $this->hasMany('App\Models\Mark', 'user_id');
    }

    public function extractions()
    {
        return $this->hasMany('App\Models\Extraction', 'student_id');
    }

    public function statement()
    {
        return $this->hasMany('App\Models\Extraction', 'student_id');
    }
    public function individual_statements()
    {
        return $this->belongsToMany('App\Models\Statement', 'individual_statement_student', 'student_id', 'statement_id');
    }

    public function supervisions()
    {
        return $this->hasMany(Supervision::class, 'student_id');
    }

    /*
    |--------------------------------------------------------------------------
    | SCOPES
    |--------------------------------------------------------------------------
    */

    /*
    |--------------------------------------------------------------------------
    | ACCESORS
    |--------------------------------------------------------------------------
    */

    public function getStudentCardAttribute()
    {
        if (!empty($this->user_attributes->where('key', 'studentCard')->first())) {
            return $this->user_attributes->where('key', 'studentCard')->first()->value ?? '-';
        }
    }

    public function getStudentGroupNameAttribute()
    {
        if(!empty($this->groups()->orderByDesc('pivot_updated_at')->first())) {
            return $this->groups()->orderByDesc('pivot_updated_at')->first()->name;
        }
    }

    public function getStudentStatusAttribute()
    {
        if (!empty($this->user_attributes) && $this->user_attributes->isNotEmpty()) {
            $user_status = $this->user_attributes->where('key', 'status')->first()->value;
        } else {
            $user_status = $this->user_attributes()->where('key', 'status')->first()->value;
        }
        return Arr::pluck(json_decode(config('settings.statuses'), true), 'name', 'slug')[$user_status] ?? $user_status;
    }

    public function getStudentStatusSlugAttribute()
    {
        if(!empty($this->user_attributes) && $this->user_attributes->isNotEmpty()){
            return $this->user_attributes->where('key', 'status')->first()->value;
        }
        return $this->user_attributes()->where('key', 'status')->first()->value;
    }

    /*
    |--------------------------------------------------------------------------
    | MUTATORS
    |--------------------------------------------------------------------------
    */
}
