<?php

namespace App\Models;

use App\Traits\HasTranslations;
use Illuminate\Database\Eloquent\Model;
use Backpack\CRUD\CrudTrait;

class Subject extends Model
{
    use CrudTrait;
    use HasTranslations;

    /*
    |--------------------------------------------------------------------------
    | GLOBAL VARIABLES
    |--------------------------------------------------------------------------
    */

    protected $table = 'subjects';
    protected $primaryKey = 'id';
    public $timestamps = true;
    protected $guarded = ['id'];
    protected $fillable = [
        'name',
        'description'
    ];
    public $translatable = [
        'name',
        'description'
    ];

    /*
    |--------------------------------------------------------------------------
    | FUNCTIONS
    |--------------------------------------------------------------------------
    */

    /*
    |--------------------------------------------------------------------------
    | RELATIONS
    |--------------------------------------------------------------------------
    */


    public function getNamesAttribute(){
        if($this->getAttribute('name') == null){
            $translated = $this->getTranslatedLocales('name');
            foreach ($translated as $lang){
                if($this->hasTranslation('name', $lang)){
                    return $this->getTranslations('name')[$lang];
                }
            }
            return '';
        }
        return $this->getAttribute('name');

    }

    public function marks()
    {
        return $this->hasMany('App\Models\Mark');
    }

    public function faculties()
    {
        return $this->belongsToMany('App\Models\Faculty', 'faculty_subject');
    }

    public function students()
    {
        return $this->belongsToMany('App\Student', 'student_subject');
    }
    /*
    |--------------------------------------------------------------------------
    | SCOPES
    |--------------------------------------------------------------------------
    */

    /*
    |--------------------------------------------------------------------------
    | ACCESORS
    |--------------------------------------------------------------------------
    */

    /*
    |--------------------------------------------------------------------------
    | MUTATORS
    |--------------------------------------------------------------------------
    */
}
